from unittest.mock import MagicMock

import httpx

from qgendapy.models.request import RequestLimit
from qgendapy.resources.request_limit import RequestLimitResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestRequestLimitList:
    def test_returns_request_limits(self):
        data = [{"RequestLimitKey": "rlk1", "Name": "Vacation Limit"}]
        client = _mock_client(data)
        resource = RequestLimitResource(client)
        resp = resource.list()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], RequestLimit)
        assert resp.items[0].name == "Vacation Limit"

    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = RequestLimitResource(client)
        resource.list()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/requestlimit"


class TestRequestLimitCRUD:
    def test_create(self):
        client = _mock_client({"RequestLimitKey": "rlk-new"})
        resource = RequestLimitResource(client)
        resource.create(data={"Name": "New Limit"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/requestlimit"

    def test_update(self):
        client = _mock_client({"RequestLimitKey": "rlk1"})
        resource = RequestLimitResource(client)
        resource.update("rlk1", data={"Name": "Updated"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/requestlimit/rlk1"

    def test_delete(self):
        client = _mock_client("")
        resource = RequestLimitResource(client)
        resource.delete("rlk1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/requestlimit/rlk1"


class TestRequestLimitStaffQuotas:
    def test_staff_quotas_path(self):
        client = _mock_client([])
        resource = RequestLimitResource(client)
        resource.staff_quotas("rlk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/requestlimit/rlk1/staffquota"

    def test_create_staff_quota(self):
        client = _mock_client({})
        resource = RequestLimitResource(client)
        resource.create_staff_quota("rlk1", data={"Quota": 5})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/requestlimit/rlk1/staffquota"

    def test_update_staff_quota(self):
        client = _mock_client({})
        resource = RequestLimitResource(client)
        resource.update_staff_quota("rlk1", "sk1", data={"Quota": 10})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/requestlimit/rlk1/staffquota/sk1"


class TestRequestLimitTaskShifts:
    def test_task_shifts_path(self):
        client = _mock_client([])
        resource = RequestLimitResource(client)
        resource.task_shifts("rlk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/requestlimit/rlk1/taskshift"

    def test_create_task_shift(self):
        client = _mock_client({})
        resource = RequestLimitResource(client)
        resource.create_task_shift("rlk1", data={"TaskKey": "tk1"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"

    def test_delete_task_shift(self):
        client = _mock_client("")
        resource = RequestLimitResource(client)
        resource.delete_task_shift("rlk1", "tsk1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/requestlimit/rlk1/taskshift/tsk1"



from unittest.mock import MagicMock

import httpx

from qgendapy.resources.support import SupportResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("POST", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestSupportSendMessage:
    def test_hits_correct_path(self):
        client = _mock_client({})
        resource = SupportResource(client)
        resource.send_message("sk1", data={"Message": "Hello"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/support/staff/sk1"

    def test_passes_json_data(self):
        client = _mock_client({})
        resource = SupportResource(client)
        resource.send_message("sk1", data={"Message": "Hello"})

        call_args = client._transport.request.call_args
        assert call_args.kwargs.get("json") == {"Message": "Hello"}



class QGendaError(Exception):
    """Base exception for all qgendapy errors."""


class AuthenticationError(QGendaError):
    """Raised when authentication fails."""


class APIError(QGendaError):
    """Raised when the QGenda API returns a 4xx/5xx response."""

    def __init__(
        self,
        status_code: int,
        message: str,
        response_body: str | dict | None = None,
    ):
        self.status_code = status_code
        self.message = message
        self.response_body = response_body
        super().__init__(f"[{status_code}] {message}")


class ConfigurationError(QGendaError):
    """Raised when required configuration is missing or invalid."""


class ODataError(QGendaError):
    """Raised when an OData query is malformed."""



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.request import ApprovedRequest, Request
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class RequestResource(BaseResource):
    """Synchronous request endpoints."""

    def list(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        include_removed: bool | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[Request]:
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        if include_removed is not None:
            params["includeRemoved"] = str(include_removed).lower()
        return self._get("/request", params=params, model=Request, odata=odata)

    def approved(
        self,
        *,
        start_date: str | None = None,
        end_date: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[ApprovedRequest]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        if start_date:
            params["startDate"] = start_date
        if end_date:
            params["endDate"] = end_date
        return self._get("/request/approved", params=params, model=ApprovedRequest, odata=odata)


class AsyncRequestResource(AsyncBaseResource):
    """Asynchronous request endpoints."""

    async def list(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        include_removed: bool | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[Request]:
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        if include_removed is not None:
            params["includeRemoved"] = str(include_removed).lower()
        return await self._get("/request", params=params, model=Request, odata=odata)

    async def approved(
        self,
        *,
        start_date: str | None = None,
        end_date: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[ApprovedRequest]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        if start_date:
            params["startDate"] = start_date
        if end_date:
            params["endDate"] = end_date
        return await self._get(
            "/request/approved", params=params, model=ApprovedRequest, odata=odata
        )



"""Basic usage of qgendapy -- fetching schedules and staff."""

from qgendapy import QGendaClient

# Credentials are loaded from the INI file pointed to by QGENDA_CONF_FILE.
# See env_config.py for details.
client = QGendaClient()

# --- Schedules ---

schedule = client.schedule.list(start_date="2024-01-15", end_date="2024-01-21")
for entry in schedule:
    print(f"{entry.staff_f_name} {entry.staff_l_name}: {entry.task_name}")

# --- Staff ---

staff = client.staff.list()
for member in staff:
    print(f"{member.first_name} {member.last_name}")

# --- Clean up ---

client.close()



"""Using OData queries to filter, select, and sort API results."""

from qgendapy import OData, QGendaClient

client = QGendaClient()

# Select specific fields only
odata = OData().select("FirstName", "LastName", "Email")
staff = client.staff.list(odata=odata)
for member in staff:
    print(f"{member.first_name} {member.last_name} <{member.email}>")

# Filter active staff
odata = OData().filter("IsActive eq true")
active_staff = client.staff.list(odata=odata)
print(f"Active staff count: {len(active_staff)}")

# Combine select, filter, and orderby
odata = (
    OData()
    .select("StaffFName", "StaffLName", "TaskName", "StartDate")
    .filter("IsPublished eq true")
    .orderby("StartDate desc")
)
schedule = client.schedule.list(start_date="2024-01-01", end_date="2024-01-31", odata=odata)
for entry in schedule:
    print(f"{entry.start_date}: {entry.staff_f_name} - {entry.task_name}")

# Expand related entities
odata = OData().expand("Tags")
tasks = client.task.list(odata=odata)
for task in tasks:
    print(f"{task.task_name} (extra fields: {task._extra})")

client.close()



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Generic, TypeVar

from qgendapy.exceptions import APIError

if TYPE_CHECKING:
    import httpx

T = TypeVar("T")


@dataclass
class QGendaResponse(Generic[T]):
    """Wrapper around a QGenda API response."""

    data: list[dict] | dict
    status_code: int
    headers: dict[str, str]
    items: list[T] = field(default_factory=list)

    def __iter__(self):
        return iter(self.items)

    def __len__(self):
        return len(self.items)

    def __bool__(self):
        return self.status_code < 400

    def __getitem__(self, index: int) -> T:
        return self.items[index]

    @classmethod
    def from_httpx(cls, resp: httpx.Response, model: type[T] | None = None) -> QGendaResponse[T]:
        """Build a QGendaResponse from an httpx.Response.

        Raises APIError for 4xx/5xx status codes.
        """
        if resp.status_code >= 400:
            try:
                body = resp.json()
            except (ValueError, TypeError):
                body = resp.text
            raise APIError(
                status_code=resp.status_code,
                message=resp.reason_phrase or "API error",
                response_body=body,
            )

        try:
            data = resp.json()
        except (ValueError, TypeError):
            data = {}

        headers = dict(resp.headers)
        items: list[T] = []

        if model is not None:
            if isinstance(data, list):
                items = [model.from_dict(item) for item in data]  # type: ignore[attr-defined]
            elif isinstance(data, dict):
                items = [model.from_dict(data)]  # type: ignore[attr-defined]

        return cls(data=data, status_code=resp.status_code, headers=headers, items=items)



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class Organization(BaseModel):
    """An organization."""

    organization_key: str = ""
    organization_name: str = ""



from __future__ import annotations

from typing import TYPE_CHECKING

import httpx

if TYPE_CHECKING:
    from qgendapy._auth import AsyncAuth, Auth


class Transport:
    """Synchronous HTTP transport wrapping httpx.Client."""

    def __init__(self, auth: Auth, base_url: str) -> None:
        self._auth = auth
        self._base_url = base_url
        self._client = httpx.Client(base_url=base_url)

    def request(
        self,
        method: str,
        path: str,
        *,
        params: dict | None = None,
        json: dict | list | None = None,
        data: dict | None = None,
        files: dict | None = None,
        headers: dict | None = None,
    ) -> httpx.Response:
        merged_headers = {**(headers or {}), "Authorization": f"Bearer {self._auth.token}"}
        return self._client.request(
            method,
            path,
            params=params,
            json=json,
            data=data,
            files=files,
            headers=merged_headers,
        )

    def close(self) -> None:
        self._client.close()


class AsyncTransport:
    """Asynchronous HTTP transport wrapping httpx.AsyncClient."""

    def __init__(self, auth: AsyncAuth, base_url: str) -> None:
        self._auth = auth
        self._base_url = base_url
        self._client = httpx.AsyncClient(base_url=base_url)

    async def request(
        self,
        method: str,
        path: str,
        *,
        params: dict | None = None,
        json: dict | list | None = None,
        data: dict | None = None,
        files: dict | None = None,
        headers: dict | None = None,
    ) -> httpx.Response:
        token = await self._auth.get_token()
        merged_headers = {**(headers or {}), "Authorization": f"Bearer {token}"}
        return await self._client.request(
            method,
            path,
            params=params,
            json=json,
            data=data,
            files=files,
            headers=merged_headers,
        )

    async def close(self) -> None:
        await self._client.aclose()



import pytest

from qgendapy.exceptions import (
    APIError,
    AuthenticationError,
    ConfigurationError,
    ODataError,
    QGendaError,
)


class TestExceptionHierarchy:
    def test_all_inherit_from_qgenda_error(self):
        assert issubclass(AuthenticationError, QGendaError)
        assert issubclass(APIError, QGendaError)
        assert issubclass(ConfigurationError, QGendaError)
        assert issubclass(ODataError, QGendaError)

    def test_qgenda_error_is_exception(self):
        assert issubclass(QGendaError, Exception)

    def test_catch_all_with_qgenda_error(self):
        with pytest.raises(QGendaError):
            raise AuthenticationError("bad creds")

        with pytest.raises(QGendaError):
            raise APIError(500, "server error")

        with pytest.raises(QGendaError):
            raise ConfigurationError("missing key")

        with pytest.raises(QGendaError):
            raise ODataError("bad filter")


class TestAPIError:
    def test_attributes(self):
        err = APIError(status_code=422, message="Validation failed", response_body={"x": 1})
        assert err.status_code == 422
        assert err.message == "Validation failed"
        assert err.response_body == {"x": 1}

    def test_str_format(self):
        err = APIError(status_code=404, message="Not Found")
        assert str(err) == "[404] Not Found"

    def test_response_body_default_none(self):
        err = APIError(status_code=500, message="Error")
        assert err.response_body is None

    def test_response_body_string(self):
        err = APIError(status_code=500, message="Error", response_body="raw text")
        assert err.response_body == "raw text"



from unittest.mock import MagicMock

import pytest

from qgendapy._auth import Auth
from qgendapy._config import QGendaConfig
from qgendapy._transport import Transport


@pytest.fixture
def mock_config():
    return QGendaConfig(email="test@example.com", password="secret", company_key="abc123")


@pytest.fixture
def mock_auth(mock_config):
    auth = MagicMock(spec=Auth)
    auth.token = "fake-token"
    return auth


@pytest.fixture
def mock_transport(mock_auth):
    transport = MagicMock(spec=Transport)
    return transport



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class CredentialingContact(BaseModel):
    """A credentialing contact."""

    contact_key: str = ""
    first_name: str = ""
    last_name: str = ""
    email: str = ""


@dataclass
class CredentialingLocation(BaseModel):
    """A credentialing location."""

    location_key: str = ""
    location_name: str = ""


@dataclass
class CredentialingProvider(BaseModel):
    """A credentialing provider."""

    staff_key: str = ""
    first_name: str = ""
    last_name: str = ""


@dataclass
class CredentialingPrivilege(BaseModel):
    """A credentialing privilege."""

    privilege_key: str = ""
    privilege_name: str = ""


@dataclass
class CredentialingRecord(BaseModel):
    """A credentialing record."""

    record_key: str = ""
    record_name: str = ""
    status: str = ""


@dataclass
class CredentialingWorkflow(BaseModel):
    """A credentialing workflow."""

    workflow_key: str = ""
    workflow_name: str = ""
    status: str = ""


@dataclass
class StaffAddress(BaseModel):
    """A staff address."""

    address_key: str = ""
    address_type: str = ""
    street: str = ""
    city: str = ""
    state: str = ""
    zip_code: str = ""


@dataclass
class StaffAppointment(BaseModel):
    """A staff appointment."""

    appointment_key: str = ""
    location_name: str = ""
    status: str = ""


@dataclass
class PayerEnrollment(BaseModel):
    """A payer enrollment."""

    enrollment_key: str = ""
    payer_name: str = ""
    status: str = ""


@dataclass
class ProfessionalAccount(BaseModel):
    """A professional account."""

    account_key: str = ""
    account_type: str = ""
    account_number: str = ""



from unittest.mock import MagicMock

import httpx

from qgendapy.models.pay import PayCode, PayPeriodAmount, PayRate
from qgendapy.resources.pay import PayCodeResource, PayPoolResource, PayRateResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestPayCodeList:
    def test_returns_pay_codes(self):
        data = [{"PayCodeKey": "pck1", "Name": "Overtime"}]
        client = _mock_client(data)
        resource = PayCodeResource(client)
        resp = resource.list()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], PayCode)
        assert resp.items[0].name == "Overtime"

    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = PayCodeResource(client)
        resource.list()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/PayCode"


class TestPayRateCRUD:
    def test_list(self):
        data = [{"PayRateKey": "prk1", "Name": "Standard"}]
        client = _mock_client(data)
        resource = PayRateResource(client)
        resp = resource.list()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], PayRate)

    def test_create(self):
        client = _mock_client({"PayRateKey": "prk-new"})
        resource = PayRateResource(client)
        resource.create(data={"Name": "New Rate"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/payrate"

    def test_update(self):
        client = _mock_client({"PayRateKey": "prk1"})
        resource = PayRateResource(client)
        resource.update("prk1", data={"Name": "Updated"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/payrate/prk1"

    def test_delete(self):
        client = _mock_client("")
        resource = PayRateResource(client)
        resource.delete("prk1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/payrate/prk1"


class TestPayPoolPeriodAmounts:
    def test_returns_period_amounts(self):
        data = [{"PayPeriodAmountKey": "ppa1", "Amount": 1000.0}]
        client = _mock_client(data)
        resource = PayPoolResource(client)
        resp = resource.period_amounts("ppt1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], PayPeriodAmount)

    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = PayPoolResource(client)
        resource.period_amounts("ppt1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/PayPoolTemplate/ppt1/PayPeriodAmount"



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class TimeEvent(BaseModel):
    """A time event record."""

    time_event_key: str = ""
    staff_key: str = ""
    staff_f_name: str = ""
    staff_l_name: str = ""
    task_name: str = ""
    task_key: str = ""
    start_date: str = ""
    end_date: str = ""
    hours: float | None = None
    comp_key: str = ""



from __future__ import annotations

import re
from dataclasses import dataclass, field, fields


def _pascal_to_snake(name: str) -> str:
    """Convert PascalCase to snake_case."""
    s = re.sub(r"([A-Z]+)([A-Z][a-z])", r"\1_\2", name)
    s = re.sub(r"([a-z0-9])([A-Z])", r"\1_\2", s)
    return s.lower()


def _build_field_map(cls: type) -> dict[str, str]:
    """Build mapping from PascalCase API keys to snake_case field names."""
    mapping: dict[str, str] = {}
    for f in fields(cls):
        if f.name.startswith("_"):
            continue
        # Convert snake_case field name back to likely PascalCase API key
        pascal = "".join(word.capitalize() for word in f.name.split("_"))
        mapping[pascal] = f.name
        mapping[f.name] = f.name  # also accept snake_case
    return mapping


@dataclass
class BaseModel:
    """Base for all QGenda data models.

    Automatically maps PascalCase API keys to snake_case dataclass fields,
    storing unrecognised keys in ``_extra``.
    """

    _extra: dict = field(default_factory=dict, repr=False)

    @classmethod
    def from_dict(cls, data: dict) -> BaseModel:
        field_map = _build_field_map(cls)
        known_fields = {f.name for f in fields(cls)}
        kwargs: dict = {}
        extra: dict = {}
        for key, value in data.items():
            snake = _pascal_to_snake(key)
            target = field_map.get(key) or field_map.get(snake)
            if target and target in known_fields and target != "_extra":
                kwargs[target] = value
            else:
                extra[key] = value
        kwargs["_extra"] = extra
        return cls(**kwargs)


@dataclass
class Tag(BaseModel):
    """A QGenda tag."""

    tag_key: str = ""
    tag_name: str = ""
    category_name: str = ""
    is_active: bool | None = None


@dataclass
class Profile(BaseModel):
    """A QGenda profile."""

    profile_key: str = ""
    profile_name: str = ""
    description: str = ""



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.common import Profile
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class ProfileResource(BaseResource):
    """Synchronous profile endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse[Profile]:
        return self._get("/profile", model=Profile, odata=odata)


class AsyncProfileResource(AsyncBaseResource):
    """Asynchronous profile endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse[Profile]:
        return await self._get("/profile", model=Profile, odata=odata)



import pytest

from qgendapy._config import QGendaConfig, resolve_config
from qgendapy.exceptions import ConfigurationError


class TestResolveConfigExplicitArgs:
    def test_all_args_provided(self):
        cfg = resolve_config(
            email="a@b.com", password="pw", company_key="ck", base_url="https://example.com"
        )
        assert cfg == QGendaConfig(
            email="a@b.com", password="pw", company_key="ck", base_url="https://example.com"
        )

    def test_default_base_url(self):
        cfg = resolve_config(email="a@b.com", password="pw", company_key="ck")
        assert cfg.base_url == "https://api.qgenda.com/v2"

    def test_missing_email_raises(self):
        with pytest.raises(ConfigurationError, match="email"):
            resolve_config(password="pw", company_key="ck")

    def test_missing_password_raises(self):
        with pytest.raises(ConfigurationError, match="password"):
            resolve_config(email="a@b.com", company_key="ck")

    def test_missing_company_key_raises(self):
        with pytest.raises(ConfigurationError, match="company_key"):
            resolve_config(email="a@b.com", password="pw")

    def test_missing_all_raises(self):
        with pytest.raises(ConfigurationError):
            resolve_config()


class TestResolveConfigEnvVars:
    def test_env_vars_used(self, monkeypatch):
        monkeypatch.setenv("QGENDA_EMAIL", "env@test.com")
        monkeypatch.setenv("QGENDA_PASSWORD", "envpw")
        monkeypatch.setenv("QGENDA_COMPANY_KEY", "envck")
        monkeypatch.setenv("QGENDA_BASE_URL", "https://env.example.com")
        cfg = resolve_config()
        assert cfg.email == "env@test.com"
        assert cfg.password == "envpw"
        assert cfg.company_key == "envck"
        assert cfg.base_url == "https://env.example.com"

    def test_explicit_args_override_env(self, monkeypatch):
        monkeypatch.setenv("QGENDA_EMAIL", "env@test.com")
        monkeypatch.setenv("QGENDA_PASSWORD", "envpw")
        monkeypatch.setenv("QGENDA_COMPANY_KEY", "envck")
        cfg = resolve_config(email="explicit@test.com", password="pw", company_key="ck")
        assert cfg.email == "explicit@test.com"

    def test_env_vars_override_ini(self, monkeypatch, tmp_path):
        ini_content = """\
[qgenda]
username = ini@test.com
password = inipw
company_key = inick
"""
        ini_file = tmp_path / "qgenda.ini"
        ini_file.write_text(ini_content)
        monkeypatch.setenv("QGENDA_CONF_FILE", str(ini_file))
        monkeypatch.setenv("QGENDA_EMAIL", "env@test.com")
        monkeypatch.delenv("QGENDA_PASSWORD", raising=False)
        monkeypatch.delenv("QGENDA_COMPANY_KEY", raising=False)
        monkeypatch.delenv("QGENDA_BASE_URL", raising=False)

        cfg = resolve_config()
        assert cfg.email == "env@test.com"  # env overrides INI
        assert cfg.password == "inipw"  # falls through to INI


class TestResolveConfigINI:
    def test_ini_file_used(self, monkeypatch, tmp_path):
        ini_content = """\
[qgenda]
username = ini@test.com
password = inipw
company_key = inick
api_url = https://ini.example.com
api_version = v2
"""
        ini_file = tmp_path / "qgenda.ini"
        ini_file.write_text(ini_content)
        monkeypatch.setenv("QGENDA_CONF_FILE", str(ini_file))

        cfg = resolve_config()
        assert cfg.email == "ini@test.com"
        assert cfg.password == "inipw"
        assert cfg.company_key == "inick"
        assert cfg.base_url == "https://ini.example.com/v2"

    def test_ini_custom_section(self, monkeypatch, tmp_path):
        ini_content = """\
[prod]
username = prod@test.com
password = prodpw
company_key = prodck
api_url = https://prod.example.com
"""
        ini_file = tmp_path / "qgenda.ini"
        ini_file.write_text(ini_content)
        monkeypatch.setenv("QGENDA_CONF_FILE", str(ini_file))
        monkeypatch.setenv("QGENDA_CONF_REGION", "prod")

        cfg = resolve_config()
        assert cfg.email == "prod@test.com"
        assert cfg.base_url == "https://prod.example.com"

    def test_explicit_args_override_ini(self, monkeypatch, tmp_path):
        ini_content = """\
[qgenda]
username = ini@test.com
password = inipw
company_key = inick
"""
        ini_file = tmp_path / "qgenda.ini"
        ini_file.write_text(ini_content)
        monkeypatch.setenv("QGENDA_CONF_FILE", str(ini_file))

        cfg = resolve_config(email="explicit@test.com", password="pw", company_key="ck")
        assert cfg.email == "explicit@test.com"
        assert cfg.password == "pw"

    def test_missing_ini_section_ignored(self, monkeypatch, tmp_path):
        ini_content = """\
[other]
username = x
"""
        ini_file = tmp_path / "qgenda.ini"
        ini_file.write_text(ini_content)
        monkeypatch.setenv("QGENDA_CONF_FILE", str(ini_file))

        with pytest.raises(ConfigurationError):
            resolve_config()



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class Facility(BaseModel):
    """A facility / location."""

    location_key: str = ""
    facility_name: str = ""
    abbrev: str = ""
    address: str = ""
    is_active: bool | None = None



"""Migrating from the legacy python-qgenda library.

The compat module provides a drop-in replacement. Change one import
and your existing code keeps working while you migrate incrementally.
"""

# Before (python-qgenda):
#
#   from qgenda.api.client import QGendaClient
#   client = QGendaClient(username="user@example.com", password="secret",
#                         company_key="abc123")
#   client.authenticate()
#   resp = client.get_schedule(start_date="2024-01-15")
#   data = resp.json()

# After (qgendapy compat layer -- change one import):
# Credentials are loaded from the INI file pointed to by QGENDA_CONF_FILE.

from qgendapy.compat import QGendaClient

client = QGendaClient()

# authenticate() is a no-op -- the modern client handles auth automatically
client.authenticate()

# Same method names, same return format
resp = client.get_schedule(start_date="2024-01-15", end_date="2024-01-21")
data = resp.json()
print(f"Schedule entries: {len(data)}")

# OData works the same way via odata_kwargs
resp = client.get_staff(odata_kwargs={"$filter": "IsActive eq true"})
staff = resp.json()
print(f"Active staff: {len(staff)}")

# Other legacy methods (require appropriate API permissions)
# resp = client.get_task()
# resp = client.get_timeevent(start_date="2024-01-15")
# resp = client.get_dailycase(start_date="2024-01-15")

# When ready, migrate to the modern API for typed models and more resources:
#
#   from qgendapy import QGendaClient, OData
#   client = QGendaClient()  # credentials from QGENDA_CONF_FILE
#   schedule = client.schedule.list(start_date="2024-01-15")
#   for entry in schedule:
#       print(entry.staff_f_name)



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class NotificationResource(BaseResource):
    """Synchronous notification list endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse:
        return self._get("/notificationlist", odata=odata)

    def create(self, *, data: dict) -> QGendaResponse:
        return self._post("/notificationlist", json=data)

    def get(self, notification_list_key: str, *, odata: OData | None = None) -> QGendaResponse:
        return self._get(f"/notificationlist/{notification_list_key}", odata=odata)

    def update(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return self._put(f"/notificationlist/{notification_list_key}", json=data)

    def delete(self, notification_list_key: str) -> QGendaResponse:
        return self._delete(f"/notificationlist/{notification_list_key}")

    def contacts(self, notification_list_key: str) -> QGendaResponse:
        return self._get(f"/notificationlist/{notification_list_key}/contact")

    def add_contact(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return self._post(f"/notificationlist/{notification_list_key}/contact", json=data)

    def update_contact(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return self._put(f"/notificationlist/{notification_list_key}/contact", json=data)

    def remove_contact(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return self._delete(f"/notificationlist/{notification_list_key}/contact", json=data)


class AsyncNotificationResource(AsyncBaseResource):
    """Asynchronous notification list endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse:
        return await self._get("/notificationlist", odata=odata)

    async def create(self, *, data: dict) -> QGendaResponse:
        return await self._post("/notificationlist", json=data)

    async def get(
        self, notification_list_key: str, *, odata: OData | None = None
    ) -> QGendaResponse:
        return await self._get(f"/notificationlist/{notification_list_key}", odata=odata)

    async def update(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return await self._put(f"/notificationlist/{notification_list_key}", json=data)

    async def delete(self, notification_list_key: str) -> QGendaResponse:
        return await self._delete(f"/notificationlist/{notification_list_key}")

    async def contacts(self, notification_list_key: str) -> QGendaResponse:
        return await self._get(f"/notificationlist/{notification_list_key}/contact")

    async def add_contact(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return await self._post(f"/notificationlist/{notification_list_key}/contact", json=data)

    async def update_contact(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return await self._put(f"/notificationlist/{notification_list_key}/contact", json=data)

    async def remove_contact(self, notification_list_key: str, *, data: dict) -> QGendaResponse:
        return await self._delete(f"/notificationlist/{notification_list_key}/contact", json=data)



from qgendapy.resources.schedule import _date_chunks


class TestDateChunks:
    def test_within_limit(self):
        chunks = _date_chunks("2024-01-01", "2024-03-01")
        assert chunks == [("2024-01-01", "2024-03-01")]

    def test_exactly_100_days(self):
        chunks = _date_chunks("2024-01-01", "2024-04-10")
        assert chunks == [("2024-01-01", "2024-04-10")]

    def test_over_100_days_splits(self):
        chunks = _date_chunks("2024-01-01", "2024-04-11")
        assert len(chunks) == 2
        assert chunks[0] == ("2024-01-01", "2024-04-10")
        assert chunks[1] == ("2024-04-11", "2024-04-11")

    def test_large_range(self):
        chunks = _date_chunks("2024-01-01", "2024-12-31")
        # 366 days -> 4 chunks
        assert len(chunks) == 4
        # Verify no gaps
        for i in range(len(chunks) - 1):
            end = chunks[i][1]
            next_start = chunks[i + 1][0]
            from datetime import date, timedelta

            e = date.fromisoformat(end)
            ns = date.fromisoformat(next_start)
            assert ns - e == timedelta(days=1)
        # Verify full coverage
        assert chunks[0][0] == "2024-01-01"
        assert chunks[-1][1] == "2024-12-31"

    def test_single_day(self):
        chunks = _date_chunks("2024-01-01", "2024-01-01")
        assert chunks == [("2024-01-01", "2024-01-01")]



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.request import RequestLimit
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class RequestLimitResource(BaseResource):
    """Synchronous request limit endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse[RequestLimit]:
        return self._get("/requestlimit", model=RequestLimit, odata=odata)

    def create(self, *, data: dict) -> QGendaResponse[RequestLimit]:
        return self._post("/requestlimit", json=data, model=RequestLimit)

    def update(self, request_limit_key: str, *, data: dict) -> QGendaResponse[RequestLimit]:
        return self._put(f"/requestlimit/{request_limit_key}", json=data, model=RequestLimit)

    def delete(self, request_limit_key: str) -> QGendaResponse:
        return self._delete(f"/requestlimit/{request_limit_key}")

    def staff_quotas(self, request_limit_key: str) -> QGendaResponse:
        return self._get(f"/requestlimit/{request_limit_key}/staffquota")

    def create_staff_quota(self, request_limit_key: str, *, data: dict) -> QGendaResponse:
        return self._post(f"/requestlimit/{request_limit_key}/staffquota", json=data)

    def update_staff_quota(
        self, request_limit_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse:
        return self._put(f"/requestlimit/{request_limit_key}/staffquota/{staff_key}", json=data)

    def task_shifts(self, request_limit_key: str) -> QGendaResponse:
        return self._get(f"/requestlimit/{request_limit_key}/taskshift")

    def create_task_shift(self, request_limit_key: str, *, data: dict) -> QGendaResponse:
        return self._post(f"/requestlimit/{request_limit_key}/taskshift", json=data)

    def update_task_shift(
        self, request_limit_key: str, task_shift_key: str, *, data: dict
    ) -> QGendaResponse:
        return self._put(
            f"/requestlimit/{request_limit_key}/taskshift/{task_shift_key}",
            json=data,
        )

    def delete_task_shift(self, request_limit_key: str, task_shift_key: str) -> QGendaResponse:
        return self._delete(f"/requestlimit/{request_limit_key}/taskshift/{task_shift_key}")


class AsyncRequestLimitResource(AsyncBaseResource):
    """Asynchronous request limit endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse[RequestLimit]:
        return await self._get("/requestlimit", model=RequestLimit, odata=odata)

    async def create(self, *, data: dict) -> QGendaResponse[RequestLimit]:
        return await self._post("/requestlimit", json=data, model=RequestLimit)

    async def update(self, request_limit_key: str, *, data: dict) -> QGendaResponse[RequestLimit]:
        return await self._put(f"/requestlimit/{request_limit_key}", json=data, model=RequestLimit)

    async def delete(self, request_limit_key: str) -> QGendaResponse:
        return await self._delete(f"/requestlimit/{request_limit_key}")

    async def staff_quotas(self, request_limit_key: str) -> QGendaResponse:
        return await self._get(f"/requestlimit/{request_limit_key}/staffquota")

    async def create_staff_quota(self, request_limit_key: str, *, data: dict) -> QGendaResponse:
        return await self._post(f"/requestlimit/{request_limit_key}/staffquota", json=data)

    async def update_staff_quota(
        self, request_limit_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse:
        return await self._put(
            f"/requestlimit/{request_limit_key}/staffquota/{staff_key}", json=data
        )

    async def task_shifts(self, request_limit_key: str) -> QGendaResponse:
        return await self._get(f"/requestlimit/{request_limit_key}/taskshift")

    async def create_task_shift(self, request_limit_key: str, *, data: dict) -> QGendaResponse:
        return await self._post(f"/requestlimit/{request_limit_key}/taskshift", json=data)

    async def update_task_shift(
        self, request_limit_key: str, task_shift_key: str, *, data: dict
    ) -> QGendaResponse:
        return await self._put(
            f"/requestlimit/{request_limit_key}/taskshift/{task_shift_key}",
            json=data,
        )

    async def delete_task_shift(
        self, request_limit_key: str, task_shift_key: str
    ) -> QGendaResponse:
        return await self._delete(f"/requestlimit/{request_limit_key}/taskshift/{task_shift_key}")



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.response import QGendaResponse


class IntegrationResource(BaseResource):
    """Synchronous integration endpoints."""

    def upload_file(
        self,
        *,
        files: dict,
        data: dict | None = None,
    ) -> QGendaResponse:
        return self._post("/Integration/File", files=files, data=data)


class AsyncIntegrationResource(AsyncBaseResource):
    """Asynchronous integration endpoints."""

    async def upload_file(
        self,
        *,
        files: dict,
        data: dict | None = None,
    ) -> QGendaResponse:
        return await self._post("/Integration/File", files=files, data=data)



"""Drop-in replacement for the legacy ``python-qgenda`` library.

Usage::

    from qgendapy.compat import QGendaClient

    client = QGendaClient(username="user@example.com", password="secret",
                          company_key="abc123")
    client.authenticate()  # no-op, modern client auto-authenticates
    resp = client.get_schedule(start_date="2024-01-01")
    data = resp.json()
"""

from __future__ import annotations

import json as _json
from typing import TYPE_CHECKING

from qgendapy.client import QGendaClient as _ModernClient
from qgendapy.odata import OData

if TYPE_CHECKING:
    from qgendapy.response import QGendaResponse

_DEFAULT_API_URL = "https://api.qgenda.com"
_DEFAULT_API_VERSION = "v2"


class _CompatResponse:
    """Mimics ``requests.Response`` for legacy code."""

    def __init__(self, qgenda_response: QGendaResponse) -> None:
        self.status_code: int = qgenda_response.status_code
        self.headers: dict[str, str] = qgenda_response.headers
        self._data = qgenda_response.data

    def json(self) -> list | dict:
        return self._data

    @property
    def text(self) -> str:
        return _json.dumps(self._data)


def _build_odata(odata_kwargs: dict | None) -> OData | None:
    """Convert legacy odata_kwargs dict to an OData builder, or None."""
    if not odata_kwargs:
        return None
    return OData.from_kwargs(odata_kwargs)


class QGendaClient:
    """Legacy-compatible client matching ``python-qgenda``'s QGendaClient API."""

    def __init__(
        self,
        username: str | None = None,
        password: str | None = None,
        company_key: str | None = None,
        api_url: str | None = None,
        api_version: str | None = None,
        headers: dict | None = None,
        raise_errors: bool = True,
        use_caching: bool = False,
        leader: bool = True,
    ) -> None:
        url = api_url or _DEFAULT_API_URL
        version = api_version or _DEFAULT_API_VERSION
        base_url = f"{url.rstrip('/')}/{version}"

        self._client = _ModernClient(
            email=username,
            password=password,
            company_key=company_key,
            base_url=base_url,
        )
        self._raise_errors = raise_errors

    # ------------------------------------------------------------------
    # Authentication
    # ------------------------------------------------------------------

    def authenticate(self) -> None:
        """No-op -- modern client auto-authenticates on first request."""

    # ------------------------------------------------------------------
    # Schedule
    # ------------------------------------------------------------------

    def get_schedule(
        self,
        start_date: str | None = None,
        end_date: str | None = None,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        if not start_date:
            raise ValueError("start_date is required for get_schedule")
        resp = self._client.schedule.list(
            start_date=start_date,
            end_date=end_date,
            odata=odata,
        )
        return _CompatResponse(resp)

    # ------------------------------------------------------------------
    # Staff
    # ------------------------------------------------------------------

    def get_staff(
        self,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        resp = self._client.staff.list(odata=odata)
        return _CompatResponse(resp)

    # ------------------------------------------------------------------
    # Facility
    # ------------------------------------------------------------------

    def get_facility(
        self,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        resp = self._client.facility.list(odata=odata)
        return _CompatResponse(resp)

    # ------------------------------------------------------------------
    # Organization
    # ------------------------------------------------------------------

    def get_organization(
        self,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
        organization_key: str | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        resp = self._client.organization.list(odata=odata)
        return _CompatResponse(resp)

    # ------------------------------------------------------------------
    # Task
    # ------------------------------------------------------------------

    def get_task(
        self,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        resp = self._client.task.list(odata=odata)
        return _CompatResponse(resp)

    # ------------------------------------------------------------------
    # Time Event
    # ------------------------------------------------------------------

    def get_timeevent(
        self,
        start_date: str | None = None,
        end_date: str | None = None,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        if not start_date:
            raise ValueError("start_date is required for get_timeevent")
        resp = self._client.time_event.list(
            start_date=start_date,
            end_date=end_date,
            odata=odata,
        )
        return _CompatResponse(resp)

    # ------------------------------------------------------------------
    # Daily Case
    # ------------------------------------------------------------------

    def get_dailycase(
        self,
        start_date: str | None = None,
        end_date: str | None = None,
        odata_kwargs: dict | None = None,
        headers: dict | None = None,
    ) -> _CompatResponse:
        odata = _build_odata(odata_kwargs)
        if not start_date:
            raise ValueError("start_date is required for get_dailycase")
        resp = self._client.daily_case.list(
            start_date=start_date,
            end_date=end_date,
            odata=odata,
        )
        return _CompatResponse(resp)



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar

from qgendapy.response import QGendaResponse

if TYPE_CHECKING:
    from qgendapy.client import AsyncQGendaClient, QGendaClient
    from qgendapy.odata import OData

T = TypeVar("T")


class BaseResource:
    """Base class for synchronous resource endpoints."""

    def __init__(self, client: QGendaClient) -> None:
        self._client = client

    def _get(
        self,
        path: str,
        *,
        params: dict | None = None,
        model: type[T] | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[T]:
        all_params = {**(params or {})}
        if odata:
            all_params.update(odata.to_params())
        resp = self._client._transport.request("GET", path, params=all_params)
        return QGendaResponse.from_httpx(resp, model=model)

    def _post(
        self,
        path: str,
        *,
        json: dict | list | None = None,
        params: dict | None = None,
        data: dict | None = None,
        files: dict | None = None,
        model: type[T] | None = None,
    ) -> QGendaResponse[T]:
        resp = self._client._transport.request(
            "POST", path, json=json, params=params, data=data, files=files
        )
        return QGendaResponse.from_httpx(resp, model=model)

    def _put(
        self,
        path: str,
        *,
        json: dict | None = None,
        params: dict | None = None,
        model: type[T] | None = None,
    ) -> QGendaResponse[T]:
        resp = self._client._transport.request("PUT", path, json=json, params=params)
        return QGendaResponse.from_httpx(resp, model=model)

    def _delete(
        self,
        path: str,
        *,
        json: dict | list | None = None,
        params: dict | None = None,
    ) -> QGendaResponse:
        resp = self._client._transport.request("DELETE", path, json=json, params=params)
        return QGendaResponse.from_httpx(resp)


class AsyncBaseResource:
    """Base class for asynchronous resource endpoints."""

    def __init__(self, client: AsyncQGendaClient) -> None:
        self._client = client

    async def _get(
        self,
        path: str,
        *,
        params: dict | None = None,
        model: type[T] | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[T]:
        all_params = {**(params or {})}
        if odata:
            all_params.update(odata.to_params())
        resp = await self._client._transport.request("GET", path, params=all_params)
        return QGendaResponse.from_httpx(resp, model=model)

    async def _post(
        self,
        path: str,
        *,
        json: dict | list | None = None,
        params: dict | None = None,
        data: dict | None = None,
        files: dict | None = None,
        model: type[T] | None = None,
    ) -> QGendaResponse[T]:
        resp = await self._client._transport.request(
            "POST", path, json=json, params=params, data=data, files=files
        )
        return QGendaResponse.from_httpx(resp, model=model)

    async def _put(
        self,
        path: str,
        *,
        json: dict | None = None,
        params: dict | None = None,
        model: type[T] | None = None,
    ) -> QGendaResponse[T]:
        resp = await self._client._transport.request("PUT", path, json=json, params=params)
        return QGendaResponse.from_httpx(resp, model=model)

    async def _delete(
        self,
        path: str,
        *,
        json: dict | list | None = None,
        params: dict | None = None,
    ) -> QGendaResponse:
        resp = await self._client._transport.request("DELETE", path, json=json, params=params)
        return QGendaResponse.from_httpx(resp)



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class Request(BaseModel):
    """A schedule request."""

    request_key: str = ""
    staff_key: str = ""
    start_date: str = ""
    end_date: str = ""
    type: str = ""
    status: str = ""
    notes: str = ""
    task_name: str = ""
    task_key: str = ""
    is_removed: bool | None = None


@dataclass
class ApprovedRequest(BaseModel):
    """An approved schedule request."""

    request_key: str = ""
    staff_key: str = ""
    start_date: str = ""
    end_date: str = ""
    type: str = ""
    status: str = ""
    notes: str = ""
    task_name: str = ""
    task_key: str = ""


@dataclass
class RequestLimit(BaseModel):
    """A request limit configuration."""

    request_limit_key: str = ""
    name: str = ""
    start_date: str = ""
    end_date: str = ""
    limit: int | None = None
    type: str = ""



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.organization import Organization
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class OrganizationResource(BaseResource):
    """Synchronous organization endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse[Organization]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        return self._get("/organization", params=params, model=Organization, odata=odata)


class AsyncOrganizationResource(AsyncBaseResource):
    """Asynchronous organization endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse[Organization]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        return await self._get("/organization", params=params, model=Organization, odata=odata)



from qgendapy._cache import CacheBackend, NullCache


class TestNullCache:
    def test_get_returns_none(self):
        cache = NullCache()
        assert cache.get("any_key") is None

    def test_set_does_not_raise(self):
        cache = NullCache()
        cache.set("key", {"data": 1}, ttl=60)  # should not raise

    def test_delete_does_not_raise(self):
        cache = NullCache()
        cache.delete("key")  # should not raise


class TestCacheBackendProtocol:
    def test_null_cache_satisfies_protocol(self):
        assert isinstance(NullCache(), CacheBackend)

    def test_custom_backend_satisfies_protocol(self):
        class MemoryCache:
            def __init__(self):
                self._store = {}

            def get(self, key: str) -> dict | list | None:
                return self._store.get(key)

            def set(self, key: str, value: dict | list, ttl: int = 0) -> None:
                self._store[key] = value

            def delete(self, key: str) -> None:
                self._store.pop(key, None)

        cache = MemoryCache()
        assert isinstance(cache, CacheBackend)

        cache.set("k", [1, 2, 3])
        assert cache.get("k") == [1, 2, 3]
        cache.delete("k")
        assert cache.get("k") is None

    def test_incomplete_class_not_protocol(self):
        class BadCache:
            def get(self, key: str):
                return None

        assert not isinstance(BadCache(), CacheBackend)



from dataclasses import dataclass
from unittest.mock import MagicMock

import httpx
import pytest

from qgendapy.exceptions import APIError
from qgendapy.models.common import BaseModel
from qgendapy.response import QGendaResponse


def _make_response(status_code=200, json_data=None, text="", reason_phrase="OK"):
    resp = MagicMock(spec=httpx.Response)
    resp.status_code = status_code
    resp.text = text
    resp.reason_phrase = reason_phrase
    resp.headers = {"content-type": "application/json"}
    if json_data is not None:
        resp.json.return_value = json_data
    else:
        resp.json.side_effect = ValueError("No JSON")
    return resp


@dataclass
class SampleModel(BaseModel):
    name: str = ""
    email: str = ""


class TestQGendaResponseFromHttpx:
    def test_list_response_without_model(self):
        resp = _make_response(json_data=[{"Name": "A"}, {"Name": "B"}])
        result = QGendaResponse.from_httpx(resp)
        assert result.status_code == 200
        assert result.data == [{"Name": "A"}, {"Name": "B"}]
        assert result.items == []

    def test_list_response_with_model(self):
        resp = _make_response(json_data=[{"Name": "Alice", "Email": "a@b.com"}])
        result = QGendaResponse.from_httpx(resp, model=SampleModel)
        assert len(result.items) == 1
        assert result.items[0].name == "Alice"
        assert result.items[0].email == "a@b.com"

    def test_dict_response_with_model(self):
        resp = _make_response(json_data={"Name": "Bob", "Email": "b@c.com"})
        result = QGendaResponse.from_httpx(resp, model=SampleModel)
        assert len(result.items) == 1
        assert result.items[0].name == "Bob"

    def test_error_status_raises_api_error(self):
        resp = _make_response(
            status_code=404, json_data={"error": "not found"}, reason_phrase="Not Found"
        )
        with pytest.raises(APIError) as exc_info:
            QGendaResponse.from_httpx(resp)
        assert exc_info.value.status_code == 404
        assert exc_info.value.response_body == {"error": "not found"}

    def test_error_status_with_non_json_body(self):
        resp = _make_response(status_code=500, reason_phrase="Internal Server Error")
        resp.json.side_effect = ValueError("not json")
        resp.text = "server error"
        with pytest.raises(APIError) as exc_info:
            QGendaResponse.from_httpx(resp)
        assert exc_info.value.status_code == 500
        assert exc_info.value.response_body == "server error"

    def test_empty_json_body(self):
        resp = _make_response(status_code=200)
        resp.json.side_effect = ValueError("empty")
        result = QGendaResponse.from_httpx(resp)
        assert result.data == {}


class TestQGendaResponseDunder:
    def test_iter(self):
        qr = QGendaResponse(data=[], status_code=200, headers={}, items=["a", "b"])
        assert list(qr) == ["a", "b"]

    def test_len(self):
        qr = QGendaResponse(data=[], status_code=200, headers={}, items=[1, 2, 3])
        assert len(qr) == 3

    def test_bool_true(self):
        qr = QGendaResponse(data=[], status_code=200, headers={})
        assert bool(qr) is True

    def test_bool_false(self):
        qr = QGendaResponse(data=[], status_code=400, headers={})
        assert bool(qr) is False



"""Configuration via environment variables, INI config file, or explicit arguments.

qgendapy resolves credentials in this order:
  1. Explicit arguments passed to QGendaClient()
  2. Environment variables
  3. INI config file pointed to by QGENDA_CONF_FILE env var
"""

from qgendapy import QGendaClient

# --- Option 1: Environment variables ---
#
# Set these in your shell or .env file:
#
#   export QGENDA_EMAIL="you@hospital.org"
#   export QGENDA_PASSWORD="your-password"
#   export QGENDA_COMPANY_KEY="your-company-key"
#
# Then create the client with no arguments:

client = QGendaClient()

# --- Option 2: INI config file ---
#
# Set the path to the config file:
#
#   export QGENDA_CONF_FILE="/path/to/qgenda.conf"
#
# INI file format (compatible with legacy python-qgenda):
#
#   [qgenda]
#   username = you@hospital.org
#   password = your-password
#   company_key = your-company-key
#   api_url = https://api.qgenda.com
#   api_version = v2
#
# Use a different section with:
#
#   export QGENDA_CONF_REGION="production"

# --- Option 3: Explicit arguments override everything ---

client = QGendaClient(
    email="override@hospital.org",
    password="override-password",
    company_key="override-key",
    base_url="https://api.qgenda.com/v2",
)

client.close()



from __future__ import annotations

from datetime import date, timedelta
from typing import TYPE_CHECKING

from qgendapy.exceptions import QGendaError
from qgendapy.models.schedule import AuditLogEntry, OpenShift, Rotation, ScheduleEntry
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse

_MAX_DAYS = 100


def _date_chunks(start: str, end: str) -> list[tuple[str, str]]:
    """Split a date range into chunks of at most _MAX_DAYS days."""
    s = date.fromisoformat(start)
    e = date.fromisoformat(end)
    if (e - s).days <= _MAX_DAYS:
        return [(start, end)]
    chunks: list[tuple[str, str]] = []
    while s <= e:
        chunk_end = min(s + timedelta(days=_MAX_DAYS), e)
        chunks.append((s.isoformat(), chunk_end.isoformat()))
        s = chunk_end + timedelta(days=1)
    return chunks


class ScheduleResource(BaseResource):
    """Synchronous schedule endpoints."""

    def list(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        includes: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[ScheduleEntry]:
        if end_date and _date_chunks(start_date, end_date) != [(start_date, end_date)]:
            return self._chunked_list(
                start_date=start_date, end_date=end_date, includes=includes, odata=odata
            )
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        if includes:
            params["includes"] = includes
        return self._get("/schedule", params=params, model=ScheduleEntry, odata=odata)

    def _chunked_list(
        self,
        *,
        start_date: str,
        end_date: str,
        includes: str | None,
        odata: OData | None,
    ) -> QGendaResponse[ScheduleEntry]:
        chunks = _date_chunks(start_date, end_date)
        combined: QGendaResponse[ScheduleEntry] | None = None
        for chunk_start, chunk_end in chunks:
            resp = self.list(
                start_date=chunk_start, end_date=chunk_end, includes=includes, odata=odata
            )
            if combined is None:
                combined = resp
            else:
                if isinstance(combined.data, list) and isinstance(resp.data, list):
                    combined.data.extend(resp.data)
                combined.items.extend(resp.items)
        if combined is None:
            raise QGendaError("No responses received from chunked schedule request")
        return combined

    def audit_log(
        self,
        *,
        schedule_start_date: str | None = None,
        schedule_end_date: str | None = None,
        since_modified_timestamp: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[AuditLogEntry]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        if schedule_start_date:
            params["scheduleStartDate"] = schedule_start_date
        if schedule_end_date:
            params["scheduleEndDate"] = schedule_end_date
        if since_modified_timestamp:
            params["sinceModifiedTimestamp"] = since_modified_timestamp
        return self._get("/schedule/auditLog", params=params, model=AuditLogEntry, odata=odata)

    def open_shifts(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        includes: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[OpenShift]:
        if end_date and _date_chunks(start_date, end_date) != [(start_date, end_date)]:
            return self._chunked_open_shifts(
                start_date=start_date, end_date=end_date, includes=includes, odata=odata
            )
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        if includes:
            params["includes"] = includes
        return self._get("/schedule/openshifts", params=params, model=OpenShift, odata=odata)

    def _chunked_open_shifts(
        self,
        *,
        start_date: str,
        end_date: str,
        includes: str | None,
        odata: OData | None,
    ) -> QGendaResponse[OpenShift]:
        chunks = _date_chunks(start_date, end_date)
        combined: QGendaResponse[OpenShift] | None = None
        for chunk_start, chunk_end in chunks:
            resp = self.open_shifts(
                start_date=chunk_start, end_date=chunk_end, includes=includes, odata=odata
            )
            if combined is None:
                combined = resp
            else:
                if isinstance(combined.data, list) and isinstance(resp.data, list):
                    combined.data.extend(resp.data)
                combined.items.extend(resp.items)
        if combined is None:
            raise QGendaError("No responses received from chunked schedule request")
        return combined

    def rotations(
        self,
        *,
        range_start_date: str | None = None,
        range_end_date: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[Rotation]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        if range_start_date:
            params["rangeStartDate"] = range_start_date
        if range_end_date:
            params["rangeEndDate"] = range_end_date
        return self._get("/schedule/rotations", params=params, model=Rotation, odata=odata)


class AsyncScheduleResource(AsyncBaseResource):
    """Asynchronous schedule endpoints."""

    async def list(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        includes: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[ScheduleEntry]:
        if end_date and _date_chunks(start_date, end_date) != [(start_date, end_date)]:
            return await self._chunked_list(
                start_date=start_date, end_date=end_date, includes=includes, odata=odata
            )
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        if includes:
            params["includes"] = includes
        return await self._get("/schedule", params=params, model=ScheduleEntry, odata=odata)

    async def _chunked_list(
        self,
        *,
        start_date: str,
        end_date: str,
        includes: str | None,
        odata: OData | None,
    ) -> QGendaResponse[ScheduleEntry]:
        chunks = _date_chunks(start_date, end_date)
        combined: QGendaResponse[ScheduleEntry] | None = None
        for chunk_start, chunk_end in chunks:
            resp = await self.list(
                start_date=chunk_start, end_date=chunk_end, includes=includes, odata=odata
            )
            if combined is None:
                combined = resp
            else:
                if isinstance(combined.data, list) and isinstance(resp.data, list):
                    combined.data.extend(resp.data)
                combined.items.extend(resp.items)
        if combined is None:
            raise QGendaError("No responses received from chunked schedule request")
        return combined

    async def audit_log(
        self,
        *,
        schedule_start_date: str | None = None,
        schedule_end_date: str | None = None,
        since_modified_timestamp: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[AuditLogEntry]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        if schedule_start_date:
            params["scheduleStartDate"] = schedule_start_date
        if schedule_end_date:
            params["scheduleEndDate"] = schedule_end_date
        if since_modified_timestamp:
            params["sinceModifiedTimestamp"] = since_modified_timestamp
        return await self._get(
            "/schedule/auditLog", params=params, model=AuditLogEntry, odata=odata
        )

    async def open_shifts(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        includes: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[OpenShift]:
        if end_date and _date_chunks(start_date, end_date) != [(start_date, end_date)]:
            return await self._chunked_open_shifts(
                start_date=start_date, end_date=end_date, includes=includes, odata=odata
            )
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        if includes:
            params["includes"] = includes
        return await self._get("/schedule/openshifts", params=params, model=OpenShift, odata=odata)

    async def _chunked_open_shifts(
        self,
        *,
        start_date: str,
        end_date: str,
        includes: str | None,
        odata: OData | None,
    ) -> QGendaResponse[OpenShift]:
        chunks = _date_chunks(start_date, end_date)
        combined: QGendaResponse[OpenShift] | None = None
        for chunk_start, chunk_end in chunks:
            resp = await self.open_shifts(
                start_date=chunk_start, end_date=chunk_end, includes=includes, odata=odata
            )
            if combined is None:
                combined = resp
            else:
                if isinstance(combined.data, list) and isinstance(resp.data, list):
                    combined.data.extend(resp.data)
                combined.items.extend(resp.items)
        if combined is None:
            raise QGendaError("No responses received from chunked schedule request")
        return combined

    async def rotations(
        self,
        *,
        range_start_date: str | None = None,
        range_end_date: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[Rotation]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        if range_start_date:
            params["rangeStartDate"] = range_start_date
        if range_end_date:
            params["rangeEndDate"] = range_end_date
        return await self._get("/schedule/rotations", params=params, model=Rotation, odata=odata)



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class StaffMember(BaseModel):
    """A staff member."""

    staff_key: str = ""
    first_name: str = ""
    last_name: str = ""
    email: str = ""
    abbrev: str = ""
    npi: str = ""
    is_active: bool | None = None
    start_date: str = ""
    end_date: str = ""
    tags: list[dict] | None = None
    phone: str = ""
    pager: str = ""
    mobile_phone: str = ""
    dept_name: str = ""


@dataclass
class StaffTag(BaseModel):
    """A tag associated with a staff member."""

    tag_key: str = ""
    tag_name: str = ""
    tag_category_name: str = ""


@dataclass
class StaffSkillset(BaseModel):
    """A skillset (task proficiency) for a staff member."""

    task_key: str = ""
    task_name: str = ""
    level: int | None = None


@dataclass
class StaffProfile(BaseModel):
    """A profile value for a staff member."""

    profile_key: str = ""
    profile_name: str = ""
    value: str = ""


@dataclass
class PayModifier(BaseModel):
    """A pay modifier for a staff member."""

    pay_modifier_key: str = ""
    staff_key: str = ""
    amount: float | None = None
    effective_date: str = ""



from unittest.mock import MagicMock, patch

import httpx
import pytest

from qgendapy._auth import AsyncAuth
from qgendapy._transport import AsyncTransport, Transport


class TestTransport:
    def test_request_adds_auth_header(self, mock_auth):
        with patch.object(httpx.Client, "request") as mock_request:
            mock_request.return_value = MagicMock(spec=httpx.Response)
            transport = Transport(auth=mock_auth, base_url="https://api.test.com")
            transport.request("GET", "/test")

            call_kwargs = mock_request.call_args
            assert call_kwargs.kwargs["headers"]["Authorization"] == "Bearer fake-token"

    def test_request_merges_custom_headers(self, mock_auth):
        with patch.object(httpx.Client, "request") as mock_request:
            mock_request.return_value = MagicMock(spec=httpx.Response)
            transport = Transport(auth=mock_auth, base_url="https://api.test.com")
            transport.request("GET", "/test", headers={"X-Custom": "value"})

            call_kwargs = mock_request.call_args
            assert call_kwargs.kwargs["headers"]["X-Custom"] == "value"
            assert call_kwargs.kwargs["headers"]["Authorization"] == "Bearer fake-token"

    def test_request_passes_params(self, mock_auth):
        with patch.object(httpx.Client, "request") as mock_request:
            mock_request.return_value = MagicMock(spec=httpx.Response)
            transport = Transport(auth=mock_auth, base_url="https://api.test.com")
            transport.request("GET", "/test", params={"key": "val"})

            call_kwargs = mock_request.call_args
            assert call_kwargs.kwargs["params"] == {"key": "val"}

    def test_request_passes_json(self, mock_auth):
        with patch.object(httpx.Client, "request") as mock_request:
            mock_request.return_value = MagicMock(spec=httpx.Response)
            transport = Transport(auth=mock_auth, base_url="https://api.test.com")
            transport.request("POST", "/test", json={"data": 1})

            call_kwargs = mock_request.call_args
            assert call_kwargs.kwargs["json"] == {"data": 1}

    def test_all_http_methods(self, mock_auth):
        with patch.object(httpx.Client, "request") as mock_request:
            mock_request.return_value = MagicMock(spec=httpx.Response)
            transport = Transport(auth=mock_auth, base_url="https://api.test.com")

            for method in ("GET", "POST", "PUT", "DELETE", "PATCH"):
                transport.request(method, "/test")

            methods_called = [call.args[0] for call in mock_request.call_args_list]
            assert methods_called == ["GET", "POST", "PUT", "DELETE", "PATCH"]

    def test_close(self, mock_auth):
        with patch.object(httpx.Client, "close") as mock_close:
            transport = Transport(auth=mock_auth, base_url="https://api.test.com")
            transport.close()
            mock_close.assert_called_once()


class TestAsyncTransport:
    @pytest.mark.asyncio
    async def test_request_adds_auth_header(self):
        mock_auth = MagicMock(spec=AsyncAuth)
        mock_auth.get_token = MagicMock(return_value="async-token")

        with patch.object(httpx.AsyncClient, "request") as mock_request:
            mock_response = MagicMock(spec=httpx.Response)

            async def async_return(*args, **kwargs):
                return mock_response

            mock_request.side_effect = async_return
            mock_auth.get_token = MagicMock(side_effect=lambda: "async-token")

            # Make get_token a coroutine
            async def get_token():
                return "async-token"

            mock_auth.get_token = get_token

            transport = AsyncTransport(auth=mock_auth, base_url="https://api.test.com")
            await transport.request("GET", "/test")

            call_kwargs = mock_request.call_args
            assert call_kwargs.kwargs["headers"]["Authorization"] == "Bearer async-token"

    @pytest.mark.asyncio
    async def test_close(self):
        mock_auth = MagicMock(spec=AsyncAuth)
        with patch.object(httpx.AsyncClient, "aclose") as mock_aclose:

            async def async_aclose(*args, **kwargs):
                return None

            mock_aclose.side_effect = async_aclose

            transport = AsyncTransport(auth=mock_auth, base_url="https://api.test.com")
            await transport.close()
            mock_aclose.assert_called_once()



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class DailyCase(BaseModel):
    """A daily case record."""

    daily_case_key: str = ""
    staff_key: str = ""
    staff_f_name: str = ""
    staff_l_name: str = ""
    task_name: str = ""
    task_key: str = ""
    start_date: str = ""
    end_date: str = ""
    case_count: int | None = None
    location_name: str = ""


@dataclass
class DailyConfiguration(BaseModel):
    """A daily configuration."""

    daily_configuration_key: str = ""
    name: str = ""
    description: str = ""
    is_active: bool | None = None


@dataclass
class Room(BaseModel):
    """A daily room."""

    room_key: str = ""
    room_name: str = ""
    location_key: str = ""
    location_name: str = ""


@dataclass
class PatientEncounter(BaseModel):
    """A patient encounter."""

    patient_encounter_key: str = ""
    daily_configuration_key: str = ""
    start_date: str = ""
    end_date: str = ""
    staff_key: str = ""
    room_key: str = ""


@dataclass
class CapacityRoomAssignment(BaseModel):
    """A capacity room assignment."""

    capacity_room_assignment_key: str = ""
    room_key: str = ""
    staff_key: str = ""
    start_date: str = ""
    end_date: str = ""



from __future__ import annotations

import configparser
import os
from dataclasses import dataclass

from qgendapy.exceptions import ConfigurationError


@dataclass
class QGendaConfig:
    email: str
    password: str
    company_key: str
    base_url: str = "https://api.qgenda.com/v2"


def _load_ini() -> dict[str, str]:
    """Load config from INI file if QGENDA_CONF_FILE is set."""
    conf_file = os.environ.get("QGENDA_CONF_FILE")
    if not conf_file:
        return {}

    section = os.environ.get("QGENDA_CONF_REGION", "qgenda")
    parser = configparser.ConfigParser()
    parser.read(conf_file)

    if not parser.has_section(section):
        return {}

    result: dict[str, str] = {}
    items = dict(parser.items(section))

    # INI uses "username" which maps to email
    if "username" in items:
        result["email"] = items["username"]
    if "password" in items:
        result["password"] = items["password"]
    if "company_key" in items:
        result["company_key"] = items["company_key"]

    # Combine api_url and api_version into base_url
    api_url = items.get("api_url", "")
    api_version = items.get("api_version", "")
    if api_url:
        base = api_url.rstrip("/")
        if api_version:
            base = f"{base}/{api_version}"
        result["base_url"] = base

    return result


def resolve_config(
    email: str | None = None,
    password: str | None = None,
    company_key: str | None = None,
    base_url: str | None = None,
) -> QGendaConfig:
    """Resolve configuration from args > env vars > INI file > defaults.

    Priority:
        1. Explicit keyword arguments
        2. Environment variables: QGENDA_EMAIL, QGENDA_PASSWORD,
           QGENDA_COMPANY_KEY, QGENDA_BASE_URL
        3. INI file pointed to by QGENDA_CONF_FILE (section from
           QGENDA_CONF_REGION, default "qgenda")
        4. Default base_url only

    Raises ConfigurationError if email, password, or company_key cannot be resolved.
    """
    ini = _load_ini()

    resolved_email = email or os.environ.get("QGENDA_EMAIL") or ini.get("email")
    resolved_password = password or os.environ.get("QGENDA_PASSWORD") or ini.get("password")
    resolved_company_key = (
        company_key or os.environ.get("QGENDA_COMPANY_KEY") or ini.get("company_key")
    )
    resolved_base_url = (
        base_url
        or os.environ.get("QGENDA_BASE_URL")
        or ini.get("base_url")
        or "https://api.qgenda.com/v2"
    )

    missing: list[str] = []
    if not resolved_email:
        missing.append("email")
    if not resolved_password:
        missing.append("password")
    if not resolved_company_key:
        missing.append("company_key")

    if missing:
        raise ConfigurationError(
            f"Missing required configuration: {', '.join(missing)}. "
            "Provide via arguments, environment variables (QGENDA_EMAIL, etc.), "
            "or an INI file (QGENDA_CONF_FILE)."
        )

    # After the missing check above, these are guaranteed to be str
    return QGendaConfig(
        email=resolved_email,  # type: ignore[arg-type]  # narrowed by missing check
        password=resolved_password,  # type: ignore[arg-type]  # narrowed by missing check
        company_key=resolved_company_key,  # type: ignore[arg-type]  # narrowed by missing check
        base_url=resolved_base_url,
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.credentialing import (
    CredentialingContact,
    CredentialingLocation,
    CredentialingPrivilege,
    CredentialingProvider,
    CredentialingRecord,
    CredentialingWorkflow,
    PayerEnrollment,
    ProfessionalAccount,
    StaffAddress,
    StaffAppointment,
)
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class CredentialingResource(BaseResource):
    """Synchronous credentialing endpoints."""

    # -- Contacts --

    def contacts(self, *, odata: OData | None = None) -> QGendaResponse[CredentialingContact]:
        return self._get("/credentialing/contacts", model=CredentialingContact, odata=odata)

    def contact_associations(
        self, contact_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/contacts/{contact_key}/associations", odata=odata)

    def update_contact_associations(self, contact_key: str, *, data: dict) -> QGendaResponse[dict]:
        return self._put(f"/credentialing/contacts/{contact_key}/associations", json=data)

    # -- Locations --

    def locations(self, *, odata: OData | None = None) -> QGendaResponse[CredentialingLocation]:
        return self._get("/credentialing/locations", model=CredentialingLocation, odata=odata)

    def get_location(
        self, location_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingLocation]:
        return self._get(
            f"/credentialing/location/{location_key}",
            model=CredentialingLocation,
            odata=odata,
        )

    def create_location(self, *, data: dict) -> QGendaResponse[CredentialingLocation]:
        return self._post("/credentialing/location", json=data, model=CredentialingLocation)

    def update_location(
        self, location_key: str, *, data: dict
    ) -> QGendaResponse[CredentialingLocation]:
        return self._put(
            f"/credentialing/location/{location_key}",
            json=data,
            model=CredentialingLocation,
        )

    def remove_location_contact(self, contact_key: str, location_key: str) -> QGendaResponse[dict]:
        return self._delete(
            f"/credentialing/location/contacts/{contact_key}/location/{location_key}"
        )

    # -- Location Staff --

    def location_staff(
        self, location_key: str, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/locations/{location_key}/staff/{staff_key}", odata=odata)

    def add_location_staff(
        self, location_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse[dict]:
        return self._post(f"/credentialing/locations/{location_key}/staff/{staff_key}", json=data)

    def update_location_staff(
        self, location_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse[dict]:
        return self._put(f"/credentialing/locations/{location_key}/staff/{staff_key}", json=data)

    # -- Providers --

    def providers(self, *, odata: OData | None = None) -> QGendaResponse[CredentialingProvider]:
        return self._get(
            "/Credentialing/Locations/Providers",  # QGenda API uses PascalCase for this endpoint
            model=CredentialingProvider,
            odata=odata,
        )

    # -- Privileges --

    def assigned_privileges(
        self, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingPrivilege]:
        return self._get(
            "/credentialing/privileges/assigned",
            model=CredentialingPrivilege,
            odata=odata,
        )

    # -- Staff sub-resources --

    def staff_addresses(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[StaffAddress]:
        return self._get(
            f"/credentialing/staff/{staff_key}/addresses",
            model=StaffAddress,
            odata=odata,
        )

    def staff_appointments(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[StaffAppointment]:
        return self._get(
            f"/credentialing/staff/{staff_key}/appointments",
            model=StaffAppointment,
            odata=odata,
        )

    def staff_files(self, staff_key: str, *, odata: OData | None = None) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/staff/{staff_key}/files", odata=odata)

    def staff_file(self, staff_key: str, *, params: dict | None = None) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/staff/{staff_key}/file", params=params)

    def staff_records(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingRecord]:
        return self._get(
            f"/credentialing/staff/{staff_key}/records",
            model=CredentialingRecord,
            odata=odata,
        )

    def staff_record_file(
        self, staff_key: str, record_key: str, file_key: str
    ) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/staff/{staff_key}/records/{record_key}/files/{file_key}")

    def staff_payer_enrollments(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[PayerEnrollment]:
        return self._get(
            f"/credentialing/staff/{staff_key}/payerEnrollments",
            model=PayerEnrollment,
            odata=odata,
        )

    def staff_privileges(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingPrivilege]:
        return self._get(
            f"/credentialing/staff/{staff_key}/privileges",
            model=CredentialingPrivilege,
            odata=odata,
        )

    def staff_professional_accounts(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[ProfessionalAccount]:
        return self._get(
            f"/credentialing/staff/{staff_key}/professionalAccounts",
            model=ProfessionalAccount,
            odata=odata,
        )

    # -- Staff Workflows --

    def staff_workflows(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/staff/{staff_key}/workflows",
            model=CredentialingWorkflow,
            odata=odata,
        )

    def staff_workflow(
        self, staff_key: str, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/staff/{staff_key}/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )

    def staff_workflows_v2(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/staff/{staff_key}/workflowsv2",
            model=CredentialingWorkflow,
            odata=odata,
        )

    # -- Workflows (global) --

    def workflows(self, *, odata: OData | None = None) -> QGendaResponse[CredentialingWorkflow]:
        return self._get("/credentialing/workflows", model=CredentialingWorkflow, odata=odata)

    def get_workflow(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )

    def workflow_entities(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/workflows/{workflow_key}/corporateEntities", odata=odata)

    def workflow_locations(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/workflows/{workflow_key}/locations", odata=odata)

    def workflow_staff(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return self._get(f"/credentialing/workflows/{workflow_key}/staff", odata=odata)

    # -- Location Workflows --

    def location_workflows(
        self, location_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/locations/{location_key}/workflows",
            model=CredentialingWorkflow,
            odata=odata,
        )

    def get_location_workflow(
        self, location_key: str, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/locations/{location_key}/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )

    # -- Corporate Entity Workflows --

    def entity_workflows(
        self, entity_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/corporateEntities/{entity_key}/workflows",
            model=CredentialingWorkflow,
            odata=odata,
        )

    def get_entity_workflow(
        self, entity_key: str, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return self._get(
            f"/credentialing/corporateEntities/{entity_key}/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )


class AsyncCredentialingResource(AsyncBaseResource):
    """Asynchronous credentialing endpoints."""

    # -- Contacts --

    async def contacts(
        self, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingContact]:
        return await self._get("/credentialing/contacts", model=CredentialingContact, odata=odata)

    async def contact_associations(
        self, contact_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(f"/credentialing/contacts/{contact_key}/associations", odata=odata)

    async def update_contact_associations(
        self, contact_key: str, *, data: dict
    ) -> QGendaResponse[dict]:
        return await self._put(f"/credentialing/contacts/{contact_key}/associations", json=data)

    # -- Locations --

    async def locations(
        self, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingLocation]:
        return await self._get(
            "/credentialing/locations", model=CredentialingLocation, odata=odata
        )

    async def get_location(
        self, location_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingLocation]:
        return await self._get(
            f"/credentialing/location/{location_key}",
            model=CredentialingLocation,
            odata=odata,
        )

    async def create_location(self, *, data: dict) -> QGendaResponse[CredentialingLocation]:
        return await self._post("/credentialing/location", json=data, model=CredentialingLocation)

    async def update_location(
        self, location_key: str, *, data: dict
    ) -> QGendaResponse[CredentialingLocation]:
        return await self._put(
            f"/credentialing/location/{location_key}",
            json=data,
            model=CredentialingLocation,
        )

    async def remove_location_contact(
        self, contact_key: str, location_key: str
    ) -> QGendaResponse[dict]:
        return await self._delete(
            f"/credentialing/location/contacts/{contact_key}/location/{location_key}"
        )

    # -- Location Staff --

    async def location_staff(
        self, location_key: str, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(
            f"/credentialing/locations/{location_key}/staff/{staff_key}", odata=odata
        )

    async def add_location_staff(
        self, location_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse[dict]:
        return await self._post(
            f"/credentialing/locations/{location_key}/staff/{staff_key}", json=data
        )

    async def update_location_staff(
        self, location_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse[dict]:
        return await self._put(
            f"/credentialing/locations/{location_key}/staff/{staff_key}", json=data
        )

    # -- Providers --

    async def providers(
        self, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingProvider]:
        return await self._get(
            "/Credentialing/Locations/Providers",  # QGenda API uses PascalCase for this endpoint
            model=CredentialingProvider,
            odata=odata,
        )

    # -- Privileges --

    async def assigned_privileges(
        self, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingPrivilege]:
        return await self._get(
            "/credentialing/privileges/assigned",
            model=CredentialingPrivilege,
            odata=odata,
        )

    # -- Staff sub-resources --

    async def staff_addresses(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[StaffAddress]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/addresses",
            model=StaffAddress,
            odata=odata,
        )

    async def staff_appointments(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[StaffAppointment]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/appointments",
            model=StaffAppointment,
            odata=odata,
        )

    async def staff_files(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(f"/credentialing/staff/{staff_key}/files", odata=odata)

    async def staff_file(
        self, staff_key: str, *, params: dict | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(f"/credentialing/staff/{staff_key}/file", params=params)

    async def staff_records(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingRecord]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/records",
            model=CredentialingRecord,
            odata=odata,
        )

    async def staff_record_file(
        self, staff_key: str, record_key: str, file_key: str
    ) -> QGendaResponse[dict]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/records/{record_key}/files/{file_key}"
        )

    async def staff_payer_enrollments(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[PayerEnrollment]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/payerEnrollments",
            model=PayerEnrollment,
            odata=odata,
        )

    async def staff_privileges(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingPrivilege]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/privileges",
            model=CredentialingPrivilege,
            odata=odata,
        )

    async def staff_professional_accounts(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[ProfessionalAccount]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/professionalAccounts",
            model=ProfessionalAccount,
            odata=odata,
        )

    # -- Staff Workflows --

    async def staff_workflows(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/workflows",
            model=CredentialingWorkflow,
            odata=odata,
        )

    async def staff_workflow(
        self, staff_key: str, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )

    async def staff_workflows_v2(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/staff/{staff_key}/workflowsv2",
            model=CredentialingWorkflow,
            odata=odata,
        )

    # -- Workflows (global) --

    async def workflows(
        self, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            "/credentialing/workflows", model=CredentialingWorkflow, odata=odata
        )

    async def get_workflow(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )

    async def workflow_entities(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(
            f"/credentialing/workflows/{workflow_key}/corporateEntities", odata=odata
        )

    async def workflow_locations(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(f"/credentialing/workflows/{workflow_key}/locations", odata=odata)

    async def workflow_staff(
        self, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[dict]:
        return await self._get(f"/credentialing/workflows/{workflow_key}/staff", odata=odata)

    # -- Location Workflows --

    async def location_workflows(
        self, location_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/locations/{location_key}/workflows",
            model=CredentialingWorkflow,
            odata=odata,
        )

    async def get_location_workflow(
        self, location_key: str, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/locations/{location_key}/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )

    # -- Corporate Entity Workflows --

    async def entity_workflows(
        self, entity_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/corporateEntities/{entity_key}/workflows",
            model=CredentialingWorkflow,
            odata=odata,
        )

    async def get_entity_workflow(
        self, entity_key: str, workflow_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[CredentialingWorkflow]:
        return await self._get(
            f"/credentialing/corporateEntities/{entity_key}/workflows/{workflow_key}",
            model=CredentialingWorkflow,
            odata=odata,
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.staff import (
    PayModifier,
    StaffMember,
    StaffProfile,
    StaffSkillset,
    StaffTag,
)
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class StaffResource(BaseResource):
    """Synchronous staff member endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse[StaffMember]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        return self._get("/staffmember", params=params, model=StaffMember, odata=odata)

    def create(self, *, data: dict) -> QGendaResponse[StaffMember]:
        return self._post("/staffmember", json=data, model=StaffMember)

    def get(self, staff_key: str, *, odata: OData | None = None) -> QGendaResponse[StaffMember]:
        return self._get(f"/staffmember/{staff_key}", model=StaffMember, odata=odata)

    def update(self, staff_key: str, *, data: dict) -> QGendaResponse[StaffMember]:
        return self._put(f"/staffmember/{staff_key}", json=data, model=StaffMember)

    def locations(self, staff_key: str) -> QGendaResponse:
        return self._get(f"/staffmember/{staff_key}/location")

    def tags(self, staff_key: str) -> QGendaResponse[StaffTag]:
        return self._get(f"/staffmember/{staff_key}/tag", model=StaffTag)

    def add_tag(self, staff_key: str, *, data: dict) -> QGendaResponse[StaffTag]:
        return self._post(f"/staffmember/{staff_key}/tag", json=data, model=StaffTag)

    def update_location_tag(
        self, staff_key: str, location_key: str, *, data: dict
    ) -> QGendaResponse:
        return self._put(f"/staffmember/{staff_key}/location/{location_key}/tag", json=data)

    def skillsets(self, staff_key: str) -> QGendaResponse[StaffSkillset]:
        return self._get(f"/staffmember/{staff_key}/skillset", model=StaffSkillset)

    def update_skillset(
        self, staff_key: str, task_key: str, *, data: dict
    ) -> QGendaResponse[StaffSkillset]:
        return self._put(
            f"/staffmember/{staff_key}/skillset/{task_key}",
            json=data,
            model=StaffSkillset,
        )

    def delete_skillset(self, staff_key: str, task_key: str) -> QGendaResponse:
        return self._delete(f"/staffmember/{staff_key}/skillset/{task_key}")

    def profiles(self, staff_key: str) -> QGendaResponse[StaffProfile]:
        return self._get(f"/staffmember/{staff_key}/profile", model=StaffProfile)

    def update_profile(
        self, staff_key: str, profile_key: str, *, data: dict
    ) -> QGendaResponse[StaffProfile]:
        return self._put(
            f"/staffmember/{staff_key}/profile/{profile_key}",
            json=data,
            model=StaffProfile,
        )

    def pay_modifiers(self, staff_key: str) -> QGendaResponse[PayModifier]:
        return self._get(f"/staffmember/{staff_key}/payModifier", model=PayModifier)

    def create_pay_modifier(self, staff_key: str, *, data: dict) -> QGendaResponse[PayModifier]:
        return self._post(f"/staffmember/{staff_key}/payModifier", json=data, model=PayModifier)

    def update_pay_modifier(
        self, staff_key: str, pay_modifier_key: str, *, data: dict
    ) -> QGendaResponse[PayModifier]:
        return self._put(
            f"/staffmember/{staff_key}/payModifier/{pay_modifier_key}",
            json=data,
            model=PayModifier,
        )

    def delete_pay_modifier(self, staff_key: str, pay_modifier_key: str) -> QGendaResponse:
        return self._delete(f"/staffmember/{staff_key}/payModifier/{pay_modifier_key}")

    def badge_id(self, staff_key: str) -> QGendaResponse:
        return self._get(f"/staffmember/{staff_key}/badgeId")

    def request_limits(self, staff_key: str) -> QGendaResponse:
        return self._get(f"/staffmember/{staff_key}/requestlimit")

    def set_manager(self, staff_key: str, *, data: dict) -> QGendaResponse:
        return self._post(f"/staffmember/{staff_key}/manager", json=data)


class AsyncStaffResource(AsyncBaseResource):
    """Asynchronous staff member endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse[StaffMember]:
        params: dict[str, str] = {"companyKey": self._client.company_key}
        return await self._get("/staffmember", params=params, model=StaffMember, odata=odata)

    async def create(self, *, data: dict) -> QGendaResponse[StaffMember]:
        return await self._post("/staffmember", json=data, model=StaffMember)

    async def get(
        self, staff_key: str, *, odata: OData | None = None
    ) -> QGendaResponse[StaffMember]:
        return await self._get(f"/staffmember/{staff_key}", model=StaffMember, odata=odata)

    async def update(self, staff_key: str, *, data: dict) -> QGendaResponse[StaffMember]:
        return await self._put(f"/staffmember/{staff_key}", json=data, model=StaffMember)

    async def locations(self, staff_key: str) -> QGendaResponse:
        return await self._get(f"/staffmember/{staff_key}/location")

    async def tags(self, staff_key: str) -> QGendaResponse[StaffTag]:
        return await self._get(f"/staffmember/{staff_key}/tag", model=StaffTag)

    async def add_tag(self, staff_key: str, *, data: dict) -> QGendaResponse[StaffTag]:
        return await self._post(f"/staffmember/{staff_key}/tag", json=data, model=StaffTag)

    async def update_location_tag(
        self, staff_key: str, location_key: str, *, data: dict
    ) -> QGendaResponse:
        return await self._put(f"/staffmember/{staff_key}/location/{location_key}/tag", json=data)

    async def skillsets(self, staff_key: str) -> QGendaResponse[StaffSkillset]:
        return await self._get(f"/staffmember/{staff_key}/skillset", model=StaffSkillset)

    async def update_skillset(
        self, staff_key: str, task_key: str, *, data: dict
    ) -> QGendaResponse[StaffSkillset]:
        return await self._put(
            f"/staffmember/{staff_key}/skillset/{task_key}",
            json=data,
            model=StaffSkillset,
        )

    async def delete_skillset(self, staff_key: str, task_key: str) -> QGendaResponse:
        return await self._delete(f"/staffmember/{staff_key}/skillset/{task_key}")

    async def profiles(self, staff_key: str) -> QGendaResponse[StaffProfile]:
        return await self._get(f"/staffmember/{staff_key}/profile", model=StaffProfile)

    async def update_profile(
        self, staff_key: str, profile_key: str, *, data: dict
    ) -> QGendaResponse[StaffProfile]:
        return await self._put(
            f"/staffmember/{staff_key}/profile/{profile_key}",
            json=data,
            model=StaffProfile,
        )

    async def pay_modifiers(self, staff_key: str) -> QGendaResponse[PayModifier]:
        return await self._get(f"/staffmember/{staff_key}/payModifier", model=PayModifier)

    async def create_pay_modifier(
        self, staff_key: str, *, data: dict
    ) -> QGendaResponse[PayModifier]:
        return await self._post(
            f"/staffmember/{staff_key}/payModifier", json=data, model=PayModifier
        )

    async def update_pay_modifier(
        self, staff_key: str, pay_modifier_key: str, *, data: dict
    ) -> QGendaResponse[PayModifier]:
        return await self._put(
            f"/staffmember/{staff_key}/payModifier/{pay_modifier_key}",
            json=data,
            model=PayModifier,
        )

    async def delete_pay_modifier(self, staff_key: str, pay_modifier_key: str) -> QGendaResponse:
        return await self._delete(f"/staffmember/{staff_key}/payModifier/{pay_modifier_key}")

    async def badge_id(self, staff_key: str) -> QGendaResponse:
        return await self._get(f"/staffmember/{staff_key}/badgeId")

    async def request_limits(self, staff_key: str) -> QGendaResponse:
        return await self._get(f"/staffmember/{staff_key}/requestlimit")

    async def set_manager(self, staff_key: str, *, data: dict) -> QGendaResponse:
        return await self._post(f"/staffmember/{staff_key}/manager", json=data)



from __future__ import annotations


class OData:
    """Builder for OData query parameters.

    Supports method chaining::

        params = OData().select("Name", "Email").filter("IsActive eq true").to_params()
    """

    def __init__(self) -> None:
        self._params: dict[str, str] = {}

    def select(self, *fields: str) -> OData:
        """Set the $select parameter."""
        self._params["$select"] = ",".join(fields)
        return self

    def filter(self, expr: str) -> OData:
        """Set the $filter parameter."""
        self._params["$filter"] = expr
        return self

    def orderby(self, expr: str) -> OData:
        """Set the $orderby parameter."""
        self._params["$orderby"] = expr
        return self

    def expand(self, expr: str) -> OData:
        """Set the $expand parameter."""
        self._params["$expand"] = expr
        return self

    def to_params(self) -> dict[str, str]:
        """Return a copy of the accumulated parameters."""
        return dict(self._params)

    @classmethod
    def from_kwargs(cls, kwargs: dict[str, str]) -> OData:
        """Create from a dict whose keys start with ``$``."""
        odata = cls()
        odata._params = {k: v for k, v in kwargs.items() if k.startswith("$")}
        return odata



import threading
from unittest.mock import MagicMock, patch

import httpx
import pytest

from qgendapy._auth import AsyncAuth, Auth
from qgendapy.exceptions import AuthenticationError


def _make_login_response(status_code=200, access_token="tok123", expires_in=3600):
    """Create a mock httpx.Response for login."""
    resp = MagicMock(spec=httpx.Response)
    resp.status_code = status_code
    resp.text = "error" if status_code != 200 else ""
    resp.json.return_value = {"access_token": access_token, "expires_in": expires_in}
    return resp


class TestAuth:
    @patch("qgendapy._auth.httpx.Client")
    def test_token_triggers_refresh(self, mock_client_cls):
        mock_client = MagicMock()
        mock_client_cls.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_cls.return_value.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = _make_login_response()

        auth = Auth(email="a@b.com", password="pw", base_url="https://api.test.com")
        token = auth.token

        assert token == "tok123"
        mock_client.post.assert_called_once()

    @patch("qgendapy._auth.httpx.Client")
    def test_token_cached_when_not_expired(self, mock_client_cls):
        mock_client = MagicMock()
        mock_client_cls.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_cls.return_value.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = _make_login_response()

        auth = Auth(email="a@b.com", password="pw", base_url="https://api.test.com")
        _ = auth.token
        _ = auth.token  # second call should not trigger refresh

        assert mock_client.post.call_count == 1

    @patch("qgendapy._auth.httpx.Client")
    def test_token_refreshes_when_expired(self, mock_client_cls):
        mock_client = MagicMock()
        mock_client_cls.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_cls.return_value.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = _make_login_response(expires_in=1)

        auth = Auth(email="a@b.com", password="pw", base_url="https://api.test.com")
        # expires_in=1 with 60s buffer means it's already "expired"
        _ = auth.token
        _ = auth.token

        assert mock_client.post.call_count == 2

    @patch("qgendapy._auth.httpx.Client")
    def test_login_failure_raises(self, mock_client_cls):
        mock_client = MagicMock()
        mock_client_cls.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_cls.return_value.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = _make_login_response(status_code=401)

        auth = Auth(email="a@b.com", password="pw", base_url="https://api.test.com")
        with pytest.raises(AuthenticationError, match="401"):
            _ = auth.token

    @patch("qgendapy._auth.httpx.Client")
    def test_thread_safety(self, mock_client_cls):
        mock_client = MagicMock()
        mock_client_cls.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_cls.return_value.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = _make_login_response()

        auth = Auth(email="a@b.com", password="pw", base_url="https://api.test.com")
        results = []

        def get_token():
            results.append(auth.token)

        threads = [threading.Thread(target=get_token) for _ in range(10)]
        for t in threads:
            t.start()
        for t in threads:
            t.join()

        assert len(results) == 10
        assert all(r == "tok123" for r in results)


def _make_async_client_mock(post_response):
    """Create a mock that works as an async context manager returning a client with .post()."""
    mock_client = MagicMock()

    async def mock_post(*args, **kwargs):
        return post_response

    mock_client.post = mock_post

    class FakeAsyncClient:
        async def __aenter__(self):
            return mock_client

        async def __aexit__(self, *args):
            pass

    return FakeAsyncClient


class TestAsyncAuth:
    @pytest.mark.asyncio
    @patch("qgendapy._auth.httpx.AsyncClient")
    async def test_get_token_triggers_refresh(self, mock_client_cls):
        mock_client_cls.side_effect = lambda: _make_async_client_mock(_make_login_response())()

        auth = AsyncAuth(email="a@b.com", password="pw", base_url="https://api.test.com")
        token = await auth.get_token()

        assert token == "tok123"

    @pytest.mark.asyncio
    @patch("qgendapy._auth.httpx.AsyncClient")
    async def test_login_failure_raises(self, mock_client_cls):
        mock_client_cls.side_effect = lambda: _make_async_client_mock(
            _make_login_response(status_code=401)
        )()

        auth = AsyncAuth(email="a@b.com", password="pw", base_url="https://api.test.com")
        with pytest.raises(AuthenticationError, match="401"):
            await auth.get_token()



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class StaffTargetResource(BaseResource):
    """Synchronous staff target endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse:
        return self._get("/stafftarget", odata=odata)

    def create(self, *, data: dict) -> QGendaResponse:
        return self._post("/stafftarget", json=data)

    def update(self, *, data: dict) -> QGendaResponse:
        return self._put("/stafftarget", json=data)

    def delete(self, *, data: dict) -> QGendaResponse:
        return self._delete("/stafftarget", json=data)

    def locations(self, staff_target_key: str) -> QGendaResponse:
        return self._get(f"/stafftarget/{staff_target_key}/location")

    def add_location(self, staff_target_key: str, location_key: str) -> QGendaResponse:
        return self._post(f"/stafftarget/{staff_target_key}/location/{location_key}")

    def remove_location(self, staff_target_key: str, location_key: str) -> QGendaResponse:
        return self._delete(f"/stafftarget/{staff_target_key}/location/{location_key}")

    def profiles(self, staff_target_key: str) -> QGendaResponse:
        return self._get(f"/stafftarget/{staff_target_key}/profile")

    def staff(self, staff_target_key: str) -> QGendaResponse:
        return self._get(f"/stafftarget/{staff_target_key}/staff")

    def add_staff(
        self, staff_target_key: str, staff_key: str, *, data: dict | None = None
    ) -> QGendaResponse:
        return self._post(f"/stafftarget/{staff_target_key}/staff/{staff_key}", json=data)

    def update_staff(self, staff_target_key: str, staff_key: str, *, data: dict) -> QGendaResponse:
        return self._put(f"/stafftarget/{staff_target_key}/staff/{staff_key}", json=data)

    def remove_staff(self, staff_target_key: str, staff_key: str) -> QGendaResponse:
        return self._delete(f"/stafftarget/{staff_target_key}/staff/{staff_key}")

    def task_shifts(self, staff_target_key: str) -> QGendaResponse:
        return self._get(f"/stafftarget/{staff_target_key}/taskshift")

    def add_task_shift(self, staff_target_key: str, *, data: dict) -> QGendaResponse:
        return self._post(f"/stafftarget/{staff_target_key}/taskshift", json=data)

    def remove_task_shift(self, staff_target_key: str, task_shift_key: str) -> QGendaResponse:
        return self._delete(f"/stafftarget/{staff_target_key}/taskshift/{task_shift_key}")


class AsyncStaffTargetResource(AsyncBaseResource):
    """Asynchronous staff target endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse:
        return await self._get("/stafftarget", odata=odata)

    async def create(self, *, data: dict) -> QGendaResponse:
        return await self._post("/stafftarget", json=data)

    async def update(self, *, data: dict) -> QGendaResponse:
        return await self._put("/stafftarget", json=data)

    async def delete(self, *, data: dict) -> QGendaResponse:
        return await self._delete("/stafftarget", json=data)

    async def locations(self, staff_target_key: str) -> QGendaResponse:
        return await self._get(f"/stafftarget/{staff_target_key}/location")

    async def add_location(self, staff_target_key: str, location_key: str) -> QGendaResponse:
        return await self._post(f"/stafftarget/{staff_target_key}/location/{location_key}")

    async def remove_location(self, staff_target_key: str, location_key: str) -> QGendaResponse:
        return await self._delete(f"/stafftarget/{staff_target_key}/location/{location_key}")

    async def profiles(self, staff_target_key: str) -> QGendaResponse:
        return await self._get(f"/stafftarget/{staff_target_key}/profile")

    async def staff(self, staff_target_key: str) -> QGendaResponse:
        return await self._get(f"/stafftarget/{staff_target_key}/staff")

    async def add_staff(
        self, staff_target_key: str, staff_key: str, *, data: dict | None = None
    ) -> QGendaResponse:
        return await self._post(f"/stafftarget/{staff_target_key}/staff/{staff_key}", json=data)

    async def update_staff(
        self, staff_target_key: str, staff_key: str, *, data: dict
    ) -> QGendaResponse:
        return await self._put(f"/stafftarget/{staff_target_key}/staff/{staff_key}", json=data)

    async def remove_staff(self, staff_target_key: str, staff_key: str) -> QGendaResponse:
        return await self._delete(f"/stafftarget/{staff_target_key}/staff/{staff_key}")

    async def task_shifts(self, staff_target_key: str) -> QGendaResponse:
        return await self._get(f"/stafftarget/{staff_target_key}/taskshift")

    async def add_task_shift(self, staff_target_key: str, *, data: dict) -> QGendaResponse:
        return await self._post(f"/stafftarget/{staff_target_key}/taskshift", json=data)

    async def remove_task_shift(
        self, staff_target_key: str, task_shift_key: str
    ) -> QGendaResponse:
        return await self._delete(f"/stafftarget/{staff_target_key}/taskshift/{task_shift_key}")



from unittest.mock import MagicMock

import httpx

from qgendapy.models.credentialing import (
    CredentialingContact,
    CredentialingLocation,
    CredentialingPrivilege,
    CredentialingProvider,
    CredentialingRecord,
    CredentialingWorkflow,
    PayerEnrollment,
    ProfessionalAccount,
    StaffAddress,
    StaffAppointment,
)
from qgendapy.odata import OData
from qgendapy.resources.credentialing import CredentialingResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestContacts:
    def test_returns_contacts(self):
        data = [{"ContactKey": "c1", "FirstName": "Jane", "LastName": "Doe"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.contacts()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingContact)
        assert resp.items[0].first_name == "Jane"

    def test_odata_support(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        odata = OData().filter("LastName eq 'Doe'")
        resource.contacts(odata=odata)

        call_args = client._transport.request.call_args
        params = call_args.kwargs.get("params") or call_args[1].get("params", {})
        assert "$filter" in params

    def test_contact_associations(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.contact_associations("c1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/contacts/c1/associations"

    def test_update_contact_associations(self):
        client = _mock_client({})
        resource = CredentialingResource(client)
        resource.update_contact_associations("c1", data={"LocationKey": "loc1"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/credentialing/contacts/c1/associations"


class TestLocations:
    def test_returns_locations(self):
        data = [{"LocationKey": "loc1", "LocationName": "Hospital A"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.locations()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingLocation)
        assert resp.items[0].location_name == "Hospital A"

    def test_get_location(self):
        data = {"LocationKey": "loc1", "LocationName": "Hospital A"}
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.get_location("loc1")

        assert len(resp.items) == 1
        assert resp.items[0].location_key == "loc1"

    def test_create_location(self):
        client = _mock_client({"LocationKey": "loc-new"})
        resource = CredentialingResource(client)
        resource.create_location(data={"LocationName": "New Hospital"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/credentialing/location"

    def test_update_location(self):
        client = _mock_client({"LocationKey": "loc1"})
        resource = CredentialingResource(client)
        resource.update_location("loc1", data={"LocationName": "Updated"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/credentialing/location/loc1"

    def test_remove_location_contact(self):
        client = _mock_client("")
        resource = CredentialingResource(client)
        resource.remove_location_contact("c1", "loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/credentialing/location/contacts/c1/location/loc1"


class TestLocationStaff:
    def test_location_staff(self):
        client = _mock_client({})
        resource = CredentialingResource(client)
        resource.location_staff("loc1", "s1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/locations/loc1/staff/s1"

    def test_add_location_staff(self):
        client = _mock_client({})
        resource = CredentialingResource(client)
        resource.add_location_staff("loc1", "s1", data={"Status": "Active"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/credentialing/locations/loc1/staff/s1"

    def test_update_location_staff(self):
        client = _mock_client({})
        resource = CredentialingResource(client)
        resource.update_location_staff("loc1", "s1", data={"Status": "Inactive"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"


class TestProviders:
    def test_returns_providers(self):
        data = [{"StaffKey": "s1", "FirstName": "Dr.", "LastName": "Smith"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.providers()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingProvider)


class TestPrivileges:
    def test_returns_assigned_privileges(self):
        data = [{"PrivilegeKey": "p1", "PrivilegeName": "Radiology"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.assigned_privileges()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingPrivilege)


class TestStaffSubResources:
    def test_staff_addresses(self):
        data = [{"AddressKey": "a1", "City": "Chicago", "State": "IL"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_addresses("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], StaffAddress)
        assert resp.items[0].city == "Chicago"

    def test_staff_appointments(self):
        data = [{"AppointmentKey": "ap1", "LocationName": "Hospital A"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_appointments("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], StaffAppointment)

    def test_staff_files(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.staff_files("s1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/staff/s1/files"

    def test_staff_file(self):
        client = _mock_client({})
        resource = CredentialingResource(client)
        resource.staff_file("s1", params={"fileKey": "f1"})

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/staff/s1/file"

    def test_staff_records(self):
        data = [{"RecordKey": "r1", "RecordName": "License", "Status": "Active"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_records("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingRecord)

    def test_staff_record_file(self):
        client = _mock_client({})
        resource = CredentialingResource(client)
        resource.staff_record_file("s1", "r1", "f1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/staff/s1/records/r1/files/f1"

    def test_staff_payer_enrollments(self):
        data = [{"EnrollmentKey": "e1", "PayerName": "Aetna", "Status": "Active"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_payer_enrollments("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], PayerEnrollment)

    def test_staff_privileges(self):
        data = [{"PrivilegeKey": "p1"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_privileges("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingPrivilege)

    def test_staff_professional_accounts(self):
        data = [{"AccountKey": "ac1", "AccountType": "DEA", "AccountNumber": "12345"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_professional_accounts("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], ProfessionalAccount)


class TestWorkflows:
    def test_staff_workflows(self):
        data = [{"WorkflowKey": "w1", "WorkflowName": "Onboarding", "Status": "Active"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_workflows("s1")

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingWorkflow)

    def test_staff_workflow(self):
        data = {"WorkflowKey": "w1", "WorkflowName": "Onboarding"}
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.staff_workflow("s1", "w1")

        assert len(resp.items) == 1
        assert resp.items[0].workflow_key == "w1"

    def test_staff_workflows_v2(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.staff_workflows_v2("s1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/staff/s1/workflowsv2"

    def test_global_workflows(self):
        data = [{"WorkflowKey": "w1"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.workflows()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], CredentialingWorkflow)

    def test_get_workflow(self):
        client = _mock_client({"WorkflowKey": "w1"})
        resource = CredentialingResource(client)
        resource.get_workflow("w1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/workflows/w1"

    def test_workflow_entities(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.workflow_entities("w1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/workflows/w1/corporateEntities"

    def test_workflow_locations(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.workflow_locations("w1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/workflows/w1/locations"

    def test_workflow_staff(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.workflow_staff("w1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/workflows/w1/staff"

    def test_location_workflows(self):
        data = [{"WorkflowKey": "w1"}]
        client = _mock_client(data)
        resource = CredentialingResource(client)
        resp = resource.location_workflows("loc1")

        assert len(resp.items) == 1

    def test_get_location_workflow(self):
        client = _mock_client({"WorkflowKey": "w1"})
        resource = CredentialingResource(client)
        resource.get_location_workflow("loc1", "w1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/locations/loc1/workflows/w1"

    def test_entity_workflows(self):
        client = _mock_client([])
        resource = CredentialingResource(client)
        resource.entity_workflows("ce1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/corporateEntities/ce1/workflows"

    def test_get_entity_workflow(self):
        client = _mock_client({"WorkflowKey": "w1"})
        resource = CredentialingResource(client)
        resource.get_entity_workflow("ce1", "w1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/credentialing/corporateEntities/ce1/workflows/w1"



from __future__ import annotations

from dataclasses import dataclass

from qgendapy.models.common import BaseModel


@dataclass
class Task(BaseModel):
    """A schedulable task."""

    task_key: str = ""
    task_name: str = ""
    abbrev: str = ""
    is_active: bool | None = None
    start_date: str = ""
    end_date: str = ""
    department: str = ""


@dataclass
class TaskShift(BaseModel):
    """A shift definition within a task."""

    task_shift_key: str = ""
    task_key: str = ""
    shift_name: str = ""
    start_time: str = ""
    end_time: str = ""



from unittest.mock import MagicMock

import httpx

from qgendapy.models.facility import Facility
from qgendapy.odata import OData
from qgendapy.resources.facility import FacilityResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestFacilityList:
    def test_returns_facilities(self):
        data = [{"LocationKey": "loc1", "FacilityName": "Main Hospital"}]
        client = _mock_client(data)
        resource = FacilityResource(client)
        resp = resource.list()

        assert len(resp.items) == 1
        assert isinstance(resp.items[0], Facility)
        assert resp.items[0].facility_name == "Main Hospital"

    def test_uses_location_path(self):
        client = _mock_client([])
        resource = FacilityResource(client)
        resource.list()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/location"

    def test_odata_support(self):
        client = _mock_client([])
        resource = FacilityResource(client)
        odata = OData().filter("IsActive eq true")
        resource.list(odata=odata)

        call_args = client._transport.request.call_args
        params = call_args.kwargs.get("params") or call_args[1].get("params", {})
        assert "$filter" in params


class TestFacilityGet:
    def test_gets_single(self):
        data = {"LocationKey": "loc1", "FacilityName": "Main"}
        client = _mock_client(data)
        resource = FacilityResource(client)
        resp = resource.get("loc1")

        assert resp.items[0].location_key == "loc1"


class TestFacilityCRUD:
    def test_create(self):
        client = _mock_client({"LocationKey": "loc-new"})
        resource = FacilityResource(client)
        resource.create(data={"FacilityName": "New"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"

    def test_update(self):
        client = _mock_client({"LocationKey": "loc1"})
        resource = FacilityResource(client)
        resource.update("loc1", data={"FacilityName": "Updated"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/location/loc1"

    def test_delete(self):
        client = _mock_client("")
        resource = FacilityResource(client)
        resource.delete("loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/location/loc1"


class TestFacilityStaff:
    def test_staff(self):
        client = _mock_client([])
        resource = FacilityResource(client)
        resource.staff("loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/location/loc1/staff"

    def test_add_staff(self):
        client = _mock_client({})
        resource = FacilityResource(client)
        resource.add_staff("loc1", data={"StaffKey": "s1"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"

    def test_remove_staff(self):
        client = _mock_client("")
        resource = FacilityResource(client)
        resource.remove_staff("loc1", "s1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/location/loc1/staff/s1"

    def test_staff_fte(self):
        client = _mock_client({})
        resource = FacilityResource(client)
        resource.staff_fte("loc1", "s1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/location/loc1/staff/s1/FTE"


class TestFacilityTasks:
    def test_tasks(self):
        client = _mock_client([])
        resource = FacilityResource(client)
        resource.tasks("loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/location/loc1/tasks"

    def test_add_task(self):
        client = _mock_client({})
        resource = FacilityResource(client)
        resource.add_task("loc1", data={"TaskKey": "t1"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/location/loc1/task"

    def test_remove_task(self):
        client = _mock_client("")
        resource = FacilityResource(client)
        resource.remove_task("loc1", "t1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/location/loc1/task/t1"



from __future__ import annotations

from typing import Protocol, runtime_checkable


@runtime_checkable
class CacheBackend(Protocol):
    """Protocol that cache backends must implement."""

    def get(self, key: str) -> dict | list | None: ...

    def set(self, key: str, value: dict | list, ttl: int = 0) -> None: ...

    def delete(self, key: str) -> None: ...


class NullCache:
    """No-op cache backend (default)."""

    def get(self, key: str) -> None:
        return None

    def set(self, key: str, value: dict | list, ttl: int = 0) -> None:
        pass

    def delete(self, key: str) -> None:
        pass



from importlib.metadata import version

from qgendapy.client import AsyncQGendaClient, QGendaClient
from qgendapy.odata import OData

__version__ = version("qgendapy")

__all__ = ["AsyncQGendaClient", "OData", "QGendaClient", "__version__"]



from unittest.mock import MagicMock

import httpx

from qgendapy.resources.user import UserResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestUserList:
    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = UserResource(client)
        resource.list()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/user"


class TestUserGet:
    def test_hits_correct_path(self):
        client = _mock_client({})
        resource = UserResource(client)
        resource.get("uk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/user/uk1"


class TestNonScheduledUser:
    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = UserResource(client)
        resource.non_scheduled()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/NonScheduledUser"

    def test_permissions_path(self):
        client = _mock_client([])
        resource = UserResource(client)
        resource.non_scheduled_permissions("loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/NonScheduledUser/location/loc1/permissions"



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.common import Tag
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class TagResource(BaseResource):
    """Synchronous tag endpoints."""

    def list(self, *, odata: OData | None = None) -> QGendaResponse[Tag]:
        return self._get("/tags", model=Tag, odata=odata)


class AsyncTagResource(AsyncBaseResource):
    """Asynchronous tag endpoints."""

    async def list(self, *, odata: OData | None = None) -> QGendaResponse[Tag]:
        return await self._get("/tags", model=Tag, odata=odata)



"""Handling errors from the QGenda API."""

from qgendapy import QGendaClient
from qgendapy.exceptions import APIError, AuthenticationError, ConfigurationError

# --- Configuration errors ---

try:
    # Missing credentials will raise immediately
    client = QGendaClient()
except ConfigurationError as e:
    print(f"Config error: {e}")
    print("Set QGENDA_CONF_FILE to point to your qgenda.conf file")

# --- Authentication errors ---

try:
    # Assumes QGENDA_CONF_FILE is set but credentials are invalid
    client = QGendaClient()
    client.staff.list()
except AuthenticationError as e:
    print(f"Auth failed: {e}")

# --- API errors ---

try:
    client = QGendaClient()
    client.staff.get("nonexistent-key")
except APIError as e:
    print(f"API error [{e.status_code}]: {e.message}")
    print(f"Response body: {e.response_body}")



from __future__ import annotations

from typing import TYPE_CHECKING

from qgendapy.models.time_event import TimeEvent
from qgendapy.resources._base import AsyncBaseResource, BaseResource

if TYPE_CHECKING:
    from qgendapy.odata import OData
    from qgendapy.response import QGendaResponse


class TimeEventResource(BaseResource):
    """Synchronous time event endpoints."""

    def list(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[TimeEvent]:
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        return self._get("/timeevent", params=params, model=TimeEvent, odata=odata)

    def create(self, *, data: dict) -> QGendaResponse[TimeEvent]:
        return self._post("/timeevent", json=data, model=TimeEvent)

    def update(self, time_event_key: str, *, data: dict) -> QGendaResponse[TimeEvent]:
        return self._put(f"/timeevent/{time_event_key}", json=data, model=TimeEvent)

    def delete(self, time_event_key: str) -> QGendaResponse:
        return self._delete(f"/timeevent/{time_event_key}")


class AsyncTimeEventResource(AsyncBaseResource):
    """Asynchronous time event endpoints."""

    async def list(
        self,
        *,
        start_date: str,
        end_date: str | None = None,
        odata: OData | None = None,
    ) -> QGendaResponse[TimeEvent]:
        params: dict[str, str] = {
            "startDate": start_date,
            "companyKey": self._client.company_key,
        }
        if end_date:
            params["endDate"] = end_date
        return await self._get("/timeevent", params=params, model=TimeEvent, odata=odata)

    async def create(self, *, data: dict) -> QGendaResponse[TimeEvent]:
        return await self._post("/timeevent", json=data, model=TimeEvent)

    async def update(self, time_event_key: str, *, data: dict) -> QGendaResponse[TimeEvent]:
        return await self._put(f"/timeevent/{time_event_key}", json=data, model=TimeEvent)

    async def delete(self, time_event_key: str) -> QGendaResponse:
        return await self._delete(f"/timeevent/{time_event_key}")



from unittest.mock import MagicMock

import httpx

from qgendapy.resources.notification import NotificationResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestNotificationList:
    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = NotificationResource(client)
        resource.list()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/notificationlist"


class TestNotificationCRUD:
    def test_create(self):
        client = _mock_client({})
        resource = NotificationResource(client)
        resource.create(data={"Name": "My List"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/notificationlist"

    def test_get(self):
        client = _mock_client({})
        resource = NotificationResource(client)
        resource.get("nlk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/notificationlist/nlk1"

    def test_update(self):
        client = _mock_client({})
        resource = NotificationResource(client)
        resource.update("nlk1", data={"Name": "Updated"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"
        assert call_args[0][1] == "/notificationlist/nlk1"

    def test_delete(self):
        client = _mock_client("")
        resource = NotificationResource(client)
        resource.delete("nlk1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/notificationlist/nlk1"


class TestNotificationContacts:
    def test_contacts_path(self):
        client = _mock_client([])
        resource = NotificationResource(client)
        resource.contacts("nlk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/notificationlist/nlk1/contact"

    def test_add_contact(self):
        client = _mock_client({})
        resource = NotificationResource(client)
        resource.add_contact("nlk1", data={"Email": "test@test.com"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/notificationlist/nlk1/contact"

    def test_update_contact(self):
        client = _mock_client({})
        resource = NotificationResource(client)
        resource.update_contact("nlk1", data={"Email": "new@test.com"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"

    def test_remove_contact(self):
        client = _mock_client("")
        resource = NotificationResource(client)
        resource.remove_contact("nlk1", data={"Email": "test@test.com"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/notificationlist/nlk1/contact"



from unittest.mock import MagicMock

import httpx

from qgendapy.resources.staff_target import StaffTargetResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("GET", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestStaffTargetList:
    def test_hits_correct_path(self):
        client = _mock_client([])
        resource = StaffTargetResource(client)
        resource.list()

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/stafftarget"


class TestStaffTargetCRUD:
    def test_create(self):
        client = _mock_client({})
        resource = StaffTargetResource(client)
        resource.create(data={"Name": "Target"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/stafftarget"

    def test_update(self):
        client = _mock_client({})
        resource = StaffTargetResource(client)
        resource.update(data={"Name": "Updated"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "PUT"

    def test_delete_body_based(self):
        client = _mock_client("")
        resource = StaffTargetResource(client)
        resource.delete(data={"StaffTargetKey": "stk1"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/stafftarget"


class TestStaffTargetLocations:
    def test_locations_path(self):
        client = _mock_client([])
        resource = StaffTargetResource(client)
        resource.locations("stk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/stafftarget/stk1/location"

    def test_add_location(self):
        client = _mock_client({})
        resource = StaffTargetResource(client)
        resource.add_location("stk1", "loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/stafftarget/stk1/location/loc1"

    def test_remove_location(self):
        client = _mock_client("")
        resource = StaffTargetResource(client)
        resource.remove_location("stk1", "loc1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/stafftarget/stk1/location/loc1"


class TestStaffTargetStaff:
    def test_staff_path(self):
        client = _mock_client([])
        resource = StaffTargetResource(client)
        resource.staff("stk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/stafftarget/stk1/staff"

    def test_add_staff(self):
        client = _mock_client({})
        resource = StaffTargetResource(client)
        resource.add_staff("stk1", "sk1", data={"Weight": 1.0})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/stafftarget/stk1/staff/sk1"

    def test_remove_staff(self):
        client = _mock_client("")
        resource = StaffTargetResource(client)
        resource.remove_staff("stk1", "sk1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"


class TestStaffTargetTaskShifts:
    def test_task_shifts_path(self):
        client = _mock_client([])
        resource = StaffTargetResource(client)
        resource.task_shifts("stk1")

        call_args = client._transport.request.call_args
        assert call_args[0][1] == "/stafftarget/stk1/taskshift"

    def test_add_task_shift(self):
        client = _mock_client({})
        resource = StaffTargetResource(client)
        resource.add_task_shift("stk1", data={"TaskKey": "tk1"})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"

    def test_remove_task_shift(self):
        client = _mock_client("")
        resource = StaffTargetResource(client)
        resource.remove_task_shift("stk1", "tsk1")

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "DELETE"
        assert call_args[0][1] == "/stafftarget/stk1/taskshift/tsk1"



from qgendapy.resources._base import AsyncBaseResource, BaseResource
from qgendapy.resources.company import AsyncCompanyResource, CompanyResource
from qgendapy.resources.credentialing import (
    AsyncCredentialingResource,
    CredentialingResource,
)
from qgendapy.resources.credit import (
    AsyncCreditAllocationResource,
    CreditAllocationResource,
)
from qgendapy.resources.daily import AsyncDailyResource, DailyResource
from qgendapy.resources.daily_case import AsyncDailyCaseResource, DailyCaseResource
from qgendapy.resources.facility import AsyncFacilityResource, FacilityResource
from qgendapy.resources.integration import (
    AsyncIntegrationResource,
    IntegrationResource,
)
from qgendapy.resources.notification import (
    AsyncNotificationResource,
    NotificationResource,
)
from qgendapy.resources.organization import (
    AsyncOrganizationResource,
    OrganizationResource,
)
from qgendapy.resources.pay import (
    AsyncPayCodeResource,
    AsyncPayPoolResource,
    AsyncPayRateResource,
    PayCodeResource,
    PayPoolResource,
    PayRateResource,
)
from qgendapy.resources.profile import AsyncProfileResource, ProfileResource
from qgendapy.resources.request import AsyncRequestResource, RequestResource
from qgendapy.resources.request_limit import (
    AsyncRequestLimitResource,
    RequestLimitResource,
)
from qgendapy.resources.schedule import AsyncScheduleResource, ScheduleResource
from qgendapy.resources.staff import AsyncStaffResource, StaffResource
from qgendapy.resources.staff_target import (
    AsyncStaffTargetResource,
    StaffTargetResource,
)
from qgendapy.resources.support import AsyncSupportResource, SupportResource
from qgendapy.resources.tag import AsyncTagResource, TagResource
from qgendapy.resources.task import AsyncTaskResource, TaskResource
from qgendapy.resources.time_event import AsyncTimeEventResource, TimeEventResource
from qgendapy.resources.user import AsyncUserResource, UserResource

__all__ = [
    "AsyncBaseResource",
    "AsyncCompanyResource",
    "AsyncCredentialingResource",
    "AsyncCreditAllocationResource",
    "AsyncDailyCaseResource",
    "AsyncDailyResource",
    "AsyncFacilityResource",
    "AsyncIntegrationResource",
    "AsyncNotificationResource",
    "AsyncOrganizationResource",
    "AsyncPayCodeResource",
    "AsyncPayPoolResource",
    "AsyncPayRateResource",
    "AsyncProfileResource",
    "AsyncRequestLimitResource",
    "AsyncRequestResource",
    "AsyncScheduleResource",
    "AsyncStaffResource",
    "AsyncStaffTargetResource",
    "AsyncSupportResource",
    "AsyncTagResource",
    "AsyncTaskResource",
    "AsyncTimeEventResource",
    "AsyncUserResource",
    "BaseResource",
    "CompanyResource",
    "CredentialingResource",
    "CreditAllocationResource",
    "DailyCaseResource",
    "DailyResource",
    "FacilityResource",
    "IntegrationResource",
    "NotificationResource",
    "OrganizationResource",
    "PayCodeResource",
    "PayPoolResource",
    "PayRateResource",
    "ProfileResource",
    "RequestLimitResource",
    "RequestResource",
    "ScheduleResource",
    "StaffResource",
    "StaffTargetResource",
    "SupportResource",
    "TagResource",
    "TaskResource",
    "TimeEventResource",
    "UserResource",
]



from unittest.mock import MagicMock

import httpx

from qgendapy.resources.integration import IntegrationResource


def _mock_client(response_data, status_code=200):
    client = MagicMock()
    client.company_key = "test-key"
    resp = httpx.Response(
        status_code,
        json=response_data,
        request=httpx.Request("POST", "http://test"),
    )
    client._transport.request.return_value = resp
    return client


class TestIntegrationUploadFile:
    def test_hits_correct_path(self):
        client = _mock_client({})
        resource = IntegrationResource(client)
        resource.upload_file(files={"file": ("test.csv", b"data")})

        call_args = client._transport.request.call_args
        assert call_args[0][0] == "POST"
        assert call_args[0][1] == "/Integration/File"

    def test_passes_files_and_data(self):
        client = _mock_client({})
        resource = IntegrationResource(client)
        files = {"file": ("test.csv", b"data")}
        resource.upload_file(files=files, data={"key": "value"})

        call_args = client._transport.request.call_args
        assert call_args.kwargs.get("files") == files
        assert call_args.kwargs.get("data") == {"key": "value"}
